#include "header.hpp"

#include "ppllll04.amplitude.set.hpp"

// computes the helicity amplitudes for the single resonant process qq' -> V1 V2; resonant_V specifies which boson can become resonant
void ppllll04_amplitude_set::calculate_H2_QCD_2loop_computeVVprimeHelAmplitudesSR(int i_x, int i_a){
  static Logger logger("ppllll04_amplitude_set::calculate_H2_QCD_2loop_computeVVprimeHelAmplitudesDR");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  // propagators for the decay
  double_complex DV1;
  if (SR[i_x][i_a].V1 == V_PHOT){DV1 = ma2;}
  else if (SR[i_x][i_a].V1 == V_ZLL || SR[i_x][i_a].V1 == V_ZNN){DV1 = ma2 - msi->cM2_Z;}
  else if (SR[i_x][i_a].V1 == V_WMIN || SR[i_x][i_a].V1 == V_WPLU){DV1 = ma2 - msi->cM2_W;}
  double_complex DV2;
  if (SR[i_x][i_a].V2 == V_PHOT){DV2 = mb2;}
  else if (SR[i_x][i_a].V2 == V_ZLL || SR[i_x][i_a].V2 == V_ZNN) {DV2 = mb2 - msi->cM2_Z;}
  else if (SR[i_x][i_a].V2 == V_WMIN || SR[i_x][i_a].V2 == V_WPLU) {DV2 = mb2 - msi->cM2_W;}

  // propagator denominators for the F_V contributions
  double_complex DP = s;
  double_complex DZ = s - msi->cM2_Z;
  double_complex DW = s - msi->cM2_W;

  double_complex Q_FP_FSR[2];
  Q_FP_FSR[0] = SR[i_x][i_a].L_P / DP;
  Q_FP_FSR[1] = SR[i_x][i_a].R_P / DP;
  double_complex Q_FZ_FSR[2];
  Q_FZ_FSR[0] = SR[i_x][i_a].L_Z / DZ;
  Q_FZ_FSR[1] = SR[i_x][i_a].R_Z / DZ;
  double_complex Q_FW_FSR[2];
  Q_FW_FSR[0] = SR[i_x][i_a].L_W / DW;
  Q_FW_FSR[1] = SR[i_x][i_a].R_W / DW;

  double_complex M_FSR_undressed[2][2][2][3][4];
  for (int loop=0; loop<3; loop++) {
    // intermediate gauge boson couples to 8
    M_FSR_undressed[0][0][0][loop][1] = compute_M_FSR_7(1, 2, 5, 6, 7, 8, -1, loop);
    M_FSR_undressed[0][1][0][loop][1] = compute_M_FSR_7(1, 2, 6, 5, 7, 8, -1, loop);
    M_FSR_undressed[1][0][0][loop][1] = compute_M_FSR_7(2, 1, 5, 6, 7, 8, -1, loop);
    M_FSR_undressed[1][1][0][loop][1] = compute_M_FSR_7(2, 1, 6, 5, 7, 8, -1, loop);
    M_FSR_undressed[0][0][1][loop][1] = compute_M_FSR_7(1, 2, 5, 6, 7, 8, +1, loop);
    M_FSR_undressed[0][1][1][loop][1] = compute_M_FSR_7(1, 2, 6, 5, 7, 8, +1, loop);
    M_FSR_undressed[1][0][1][loop][1] = compute_M_FSR_7(2, 1, 5, 6, 7, 8, +1, loop);
    M_FSR_undressed[1][1][1][loop][1] = compute_M_FSR_7(2, 1, 6, 5, 7, 8, +1, loop);
    // intermediate gauge boson couples to 7
    // sign from crossing of fermion lines
    M_FSR_undressed[0][0][0][loop][0] = -compute_M_FSR_7(1, 2, 5, 6, 8, 7, +1, loop);
    M_FSR_undressed[0][1][0][loop][0] = -compute_M_FSR_7(1, 2, 6, 5, 8, 7, +1, loop);
    M_FSR_undressed[1][0][0][loop][0] = -compute_M_FSR_7(2, 1, 5, 6, 8, 7, +1, loop);
    M_FSR_undressed[1][1][0][loop][0] = -compute_M_FSR_7(2, 1, 6, 5, 8, 7, +1, loop);
    M_FSR_undressed[0][0][1][loop][0] = -compute_M_FSR_7(1, 2, 5, 6, 8, 7, -1, loop);
    M_FSR_undressed[0][1][1][loop][0] = -compute_M_FSR_7(1, 2, 6, 5, 8, 7, -1, loop);
    M_FSR_undressed[1][0][1][loop][0] = -compute_M_FSR_7(2, 1, 5, 6, 8, 7, -1, loop);
    M_FSR_undressed[1][1][1][loop][0] = -compute_M_FSR_7(2, 1, 6, 5, 8, 7, -1, loop);
    // intermediate gauge boson couples to 6
    M_FSR_undressed[0][0][0][loop][3] = compute_M_FSR_7(1, 2, 7, 8, 5, 6, -1, loop);
    M_FSR_undressed[0][0][1][loop][3] = compute_M_FSR_7(1, 2, 8, 7, 5, 6, -1, loop);
    M_FSR_undressed[1][0][0][loop][3] = compute_M_FSR_7(2, 1, 7, 8, 5, 6, -1, loop);
    M_FSR_undressed[1][0][1][loop][3] = compute_M_FSR_7(2, 1, 8, 7, 5, 6, -1, loop);
    M_FSR_undressed[0][1][0][loop][3] = compute_M_FSR_7(1, 2, 7, 8, 5, 6, +1, loop);
    M_FSR_undressed[0][1][1][loop][3] = compute_M_FSR_7(1, 2, 8, 7, 5, 6, +1, loop);
    M_FSR_undressed[1][1][0][loop][3] = compute_M_FSR_7(2, 1, 7, 8, 5, 6, +1, loop);
    M_FSR_undressed[1][1][1][loop][3] = compute_M_FSR_7(2, 1, 8, 7, 5, 6, +1, loop);
    // intermediate gauge boson couples to 5
    // sign from crossing of fermion lines
    M_FSR_undressed[0][0][0][loop][2] = -compute_M_FSR_7(1, 2, 7, 8, 6, 5, +1, loop);
    M_FSR_undressed[0][0][1][loop][2] = -compute_M_FSR_7(1, 2, 8, 7, 6, 5, +1, loop);
    M_FSR_undressed[1][0][0][loop][2] = -compute_M_FSR_7(2, 1, 7, 8, 6, 5, +1, loop);
    M_FSR_undressed[1][0][1][loop][2] = -compute_M_FSR_7(2, 1, 8, 7, 6, 5, +1, loop);
    M_FSR_undressed[0][1][0][loop][2] = -compute_M_FSR_7(1, 2, 7, 8, 6, 5, -1, loop);
    M_FSR_undressed[0][1][1][loop][2] = -compute_M_FSR_7(1, 2, 8, 7, 6, 5, -1, loop);
    M_FSR_undressed[1][1][0][loop][2] = -compute_M_FSR_7(2, 1, 7, 8, 6, 5, -1, loop);
    M_FSR_undressed[1][1][1][loop][2] = -compute_M_FSR_7(2, 1, 8, 7, 6, 5, -1, loop);
  }


//   double_complex M_dressed[2][2][2][3];
  for (int loop=0; loop<3; loop++) {
    for (int h1=0; h1<2; h1++) {
      for (int h2=0; h2<2; h2++) {
        for (int h3=0; h3<2; h3++) {
	  /*
	  for (int type=0; type<4; type++) {
	    cout << "M_FSR_undressed[" << h1 << "][" << h2 << "][" << h3 << "][" << loop << "][" << type << "] = " << M_FSR_undressed[h1][h2][h3][loop][type] << endl;
	  }
	  */
          if (SR[i_x][i_a].resonant_V == 1) {
            M_dressed_tmp[h1][h2][h3][loop]  = SR[i_x][i_a].LR56[h2] * SR[i_x][i_a].LR8_1[h3] / DV1 * (Q_FP_FSR[h1] * SR[i_x][i_a].LR_P8[h3] + Q_FZ_FSR[h1] * SR[i_x][i_a].LR_Z8[h3] + Q_FW_FSR[h1] * SR[i_x][i_a].LR_W8[h3]) * M_FSR_undressed[h1][h2][h3][loop][0];
            M_dressed_tmp[h1][h2][h3][loop]  += SR[i_x][i_a].LR56[h2] * SR[i_x][i_a].LR7_1[h3] / DV1 * (Q_FP_FSR[h1] * SR[i_x][i_a].LR_P7[h3] + Q_FZ_FSR[h1] * SR[i_x][i_a].LR_Z7[h3] + Q_FW_FSR[h1] * SR[i_x][i_a].LR_W7[h3]) * M_FSR_undressed[h1][h2][h3][loop][1];
          }
	  else {
            M_dressed_tmp[h1][h2][h3][loop] = SR[i_x][i_a].LR78[h3] * SR[i_x][i_a].LR6_2[h2] / DV2 * (Q_FP_FSR[h1] * SR[i_x][i_a].LR_P6[h2] + Q_FZ_FSR[h1] * SR[i_x][i_a].LR_Z6[h2] + Q_FW_FSR[h1] * SR[i_x][i_a].LR_W6[h2]) * M_FSR_undressed[h1][h2][h3][loop][2];
            M_dressed_tmp[h1][h2][h3][loop] += SR[i_x][i_a].LR78[h3] * SR[i_x][i_a].LR5_2[h2] / DV2 * (Q_FP_FSR[h1] * SR[i_x][i_a].LR_P5[h2] + Q_FZ_FSR[h1] * SR[i_x][i_a].LR_Z5[h2] + Q_FW_FSR[h1] * SR[i_x][i_a].LR_W5[h2]) * M_FSR_undressed[h1][h2][h3][loop][3];
          }

	  //	  cout << "M_dressed_tmp[" << h1 << "][" << h2 << "][" << h3 << "][" << loop << "] = " << M_dressed_tmp[h1][h2][h3][loop] << endl;
	  //           cout << "+FSR: " << M_dressed_tmp[h1][h2][h3][loop] << endl;

          M_dressed_tmp[h1][h2][h3][loop] *= double_complex(0,1);
        }
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

double_complex ppllll04_amplitude_set::compute_M_FSR_7(int i1, int i2, int i5, int i6, int i7, int i8, int hel, int loop){
  static Logger logger("ppllll04_amplitude_set::compute_M_FSR_7");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;
  
  /* computes the "FSR" type diagram, i.e. qq' -> ll -> llll, where the intermediate gauge boson couples to i7. The diagram where it couples to i5 can be obtained by (5,6)<->(7,8),
     the diagram where it couples to i8 by 7<->8 plus helicity flip in 78
  // note that the crossing is different compared to the "resonant" contribution. swapping helicities in the initial state is always trivial (i.e. a permutation), in the final state it is non-trivial in 78 and trivial in 56 */
  
  i5 -= 2;
  i6 -= 2;
  i7 -= 2;
  i8 -= 2;

  double_complex result = 0.;
  if (hel == -1) { // LLL
    result -= za[i5][i7] * zb[i8][i1] * (zb[i5][i6] * za[i2][i5] + zb[i7][i6] * za[i2][i7]);
  }
  else if (hel == +1) { // LLR
    result -= za[i2][i8] * zb[i7][i6] * (za[i5][i6] * zb[i6][i1] + za[i5][i7] * zb[i7][i1]);
  }
  result *= (4.0 * F[loop] / (za[i5][i6] * zb[i6][i5] + za[i5][i7] * zb[i7][i5] + za[i6][i7] * zb[i7][i6]));

  logger << LOG_DEBUG_VERBOSE << "finished - return " << result << endl;
  return result;
}


//#include "old.ppllll04.amplitude.H2.QCD.helicity.amplitudes.SR.cpp"

