#include "header.hpp"

#include "ppllll04.amplitude.set.hpp"

ppllll04_decay_DR::ppllll04_decay_DR(){
  Logger logger("ppllll04_decay_DR::ppllll04_decay_DR");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  logger << LOG_DEBUG_VERBOSE << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

ppllll04_decay_DR::~ppllll04_decay_DR(){
  Logger logger("ppllll04_decay_DR::~ppllll04_decay_DR");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  logger << LOG_DEBUG_VERBOSE << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


ppllll04_decay_DR::ppllll04_decay_DR(TypeV _V1, TypeV _V2, ppllll04_amplitude_set * asi, vector<int> permutation){
  Logger logger("ppllll04_decay_DR::ppllll04_decay_DR");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  V1 = _V1;
  V2 = _V2;
  model_set * msi = asi->msi;
  int q1 = asi->q1;

  double Nf = asi->osi->N_f;
  int switch_all_quarks_as_massless = asi->osi->user->switch_value[asi->osi->user->switch_map["all_quarks_as_massless"]];
  if (switch_all_quarks_as_massless){
    logger << LOG_INFO << "switch_all_quarks_as_massless = " << switch_all_quarks_as_massless << endl;
    Nf = 6;
  }

  
  Q_lep.resize(4);
  I_lep.resize(4);
  
  LR56.resize(2);
  LR78.resize(2);
  
  Q_A.resize(2);
  Q_B.resize(2);
  Q_C.resize(2);

  if (q1==2 || q1==4) { // u, c
    I3_q=0.5;
    e_q=2.0/3;
    /*
    I3_q = osi->msi->Iw_u;
    I3_qp = -osi->msi->Iw_u;
    e_q = osi->msi->Q_u;
    e_qp = -osi->msi->Q_u;
    */
  } else if (q1==1 || q1==3 || q1==5) { // d, s, b
    I3_q=-0.5;
    e_q=-1.0/3;
    /*
    I3_q = osi->msi->Iw_d;
    I3_qp = -osi->msi->Iw_d;
    e_q = osi->msi->Q_d;
    e_qp = -osi->msi->Q_d;
    */
  }
  if ( ((V1 == V_PHOT || V1 == V_ZLL) && (V2 == V_PHOT || V2 == V_ZLL)) || (V1 == V_WMIN && V2 == V_WPLU) || (V1 == V_WPLU && V2 == V_WMIN) || (V1 == V_ZLL && V2 == V_ZNN) || (V1 == V_ZNN && V2 == V_ZLL) || (V1 == V_PHOT && V2 == V_ZNN) || (V1 == V_ZNN && V2 == V_PHOT) ) {
    //electrically neutral final state
    e_qp = -e_q; // FIXIT: this should not be the anti-quark, but the quark charge. However, it does not contribute below
    I3_qp = -I3_q;
    total_q = 0;
  } else if ( ((V1 == V_PHOT || V1 == V_ZLL || V1 == V_ZNN) && V2 == V_WMIN) || (V1 == V_WMIN && (V2 == V_PHOT || V2 == V_ZLL || V2 == V_ZNN)) ) {
    // negatively charged final state
    e_qp = -(-1-e_q);
    I3_qp = -I3_q;
    total_q = -1;
  } else if ( ((V1 == V_PHOT || V1 == V_ZLL || V1 == V_ZNN) && V2 == V_WPLU) || (V1 == V_WPLU && (V2 == V_PHOT || V2 == V_ZLL || V2 == V_ZNN)) ) {
    // positively charged final state
    e_qp = -(1-e_q);
    I3_qp = -I3_q;
    total_q = 1;
  } else {
    assert(false);
  }


    if (V1 == V_PHOT || V1 == V_ZLL) {
    Q_lep[0]=-1;
    I_lep[0]=-0.5;
    Q_lep[1]=-1;
    I_lep[1]=-0.5;
  } else if (V1 == V_WMIN) {
    Q_lep[0]=0;
    I_lep[0]=0.5;
    Q_lep[1]=-1;
    I_lep[1]=-0.5;
  } else if (V1 == V_WPLU) {
    Q_lep[0]=-1;
    I_lep[0]=-0.5;
    Q_lep[1]=0;
    I_lep[1]=0.5;
  } else if (V1 == V_ZNN) {
    Q_lep[0]=0;
    I_lep[0]=0.5;
    Q_lep[1]=0;
    I_lep[1]=0.5;
  }
  if (V2 == V_PHOT || V2 == V_ZLL) {
    Q_lep[2]=-1;
    I_lep[2]=-0.5;
    Q_lep[3]=-1;
    I_lep[3]=-0.5;
  } else if (V2 == V_WMIN) {
    Q_lep[2]=0;
    I_lep[2]=0.5;
    Q_lep[3]=-1;
    I_lep[3]=-0.5;
  } else if (V2 == V_WPLU) {
    Q_lep[2]=-1;
    I_lep[2]=-0.5;
    Q_lep[3]=0;
    I_lep[3]=0.5;
  } else if (V2 == V_ZNN) {
    Q_lep[2]=0;
    I_lep[2]=0.5;
    Q_lep[3]=0;
    I_lep[3]=0.5;
  }


  if (V1 == V_PHOT) { //photon
    LR56[0] = -1.0;
    LR56[1] = -1.0;

    LR56[0] = Q_lep[0];
    LR56[1] = Q_lep[0];
    /*
    LR56[0] = osi->msi->Q_l;
    LR56[1] = osi->msi->Q_l;
    */
  } else if (V1 == V_ZLL || V1 == V_ZNN) { // Z // TODO: depends on the decay!
    // LR56[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    // LR56[1] = msi->csin_w/msi->ccos_w;
    LR56[0] = (I_lep[0]-Q_lep[0]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR56[1] = -Q_lep[0]*msi->csin_w/msi->ccos_w;
    /*
    LR56[0] = osi->msi->cCminus_Zee;
    LR56[1] = osi->msi->cCplus_Zee;
    // Z -> nn~ decay !!!
    LR56[0] = osi->msi->cCminus_Znn;
    LR56[1] = osi->msi->cCplus_Znn;
    */
  } else if (V1 == 2 || V1 == 3) { // W
    LR56[0] = 1.0/sqrt(2)/msi->csin_w;
    LR56[1] = 0.0;
    /*
    LR56[0] = osi->msi->cCminus_W;
    LR56[1] = 0.;
    */
  }
  if (V2 == V_PHOT) { //photon
    LR78[0] = -1.0;
    LR78[1] = -1.0;
    /*
    LR78[0] = osi->msi->Q_l;
    LR78[1] = osi->msi->Q_l;
    */
  } else if (V2 == V_ZLL || V2 == V_ZNN) { // Z
    LR78[0] = (I_lep[2]-Q_lep[2]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR78[1] = -Q_lep[2]*msi->csin_w/msi->ccos_w;
    // LR78[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    // LR78[1] = msi->csin_w/msi->ccos_w;
    /*
    LR78[0] = osi->msi->cCminus_Zee;
    LR78[1] = osi->msi->cCplus_Zee;
    // Z -> nn~ decay !!!
    LR78[0] = osi->msi->cCminus_Znn;
    LR78[1] = osi->msi->cCplus_Znn;
    */
  } else if (V2 == 2 || V2 == 3) { // W
    LR78[0] = 1.0/sqrt(2)/msi->csin_w;
    LR78[1] = 0.0;
    /*
    LR78[0] = osi->msi->cCminus_W;
    LR78[1] = 0.;
    */
  }

  c_P12 = 0;
  c_Z12 = 0;
  c_W12 = 0;
  L_P = 0.;
  L_Z = 0.;
  L_W = 0.;
  R_P = 0.;
  R_Z = 0.;
  R_W = 0.;
  
  // quark boson couplings
  if (total_q != 0) { // udb or dub
    L_W=1.0/sqrt(2)/msi->csin_w;
    R_W=0.0;
    /*
    L_W = osi->msi->cCminus_W;
    R_W = 0.;
    */
  } else { // uub or ddb
    L_P=e_q;
    R_P=e_q;
    L_Z=(I3_q-e_q*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    R_Z=-e_q*msi->csin_w/msi->ccos_w;
  }
  
  if ((V1 == V_WMIN || V1 == V_WPLU) && (V2 == V_WMIN || V2 == V_WPLU)) { // WW
    c_P12=1;
    c_Z12=msi->ccos_w/msi->csin_w;
    /*
    c_P12 = osi->msi->cC_AWminusWplus; // opposite sign compared to SD notation (all incoming) !!!
    c_Z12 = osi->msi->cC_ZWminusWplus; // same sign compared to SD notation (all incoming) !!!
    */
  } else if ((V1 == V_PHOT && V2 == V_WMIN) || (V1 == V_WMIN && V2 == V_PHOT)) { // gamma W
    c_W12=1;
  } else if ((V1 == V_PHOT && V2 == V_WPLU) || (V1 == V_WPLU && V2 == V_PHOT)) { // gamma W
    c_W12=-1;
  } else if ((V1 == V_WMIN && (V2 == V_ZLL || V2 == V_ZNN)) || ((V1 == V_ZLL || V1 == V_ZNN) && V2 == V_WMIN)) { // WZ
    c_W12=msi->ccos_w/msi->csin_w;
  } else if ((V1 == V_WPLU && (V2 == V_ZLL || V2 == V_ZNN)) || ((V1 == V_ZLL || V1 == V_ZNN) && V2 == V_WPLU)) { // WZ
    c_W12=-msi->ccos_w/msi->csin_w;
  }


  if (V1 == V_PHOT) {
      LAq1 = e_q;
      RAq1 = e_q;
    if (V2 == V_PHOT || V2 == V_ZLL || V2 == V_ZNN) {
      LBq1 = e_q;
      RBq1 = e_q;
    } else if (V2 == V_WMIN || V2 == V_WPLU) {
      LBq1 = e_qp;
      RBq1 = e_qp;
    }
  } else if (V1 == V_ZLL || V1 == V_ZNN) {
    LAq1 = (I3_q-e_q*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    RAq1 = -e_q*msi->csin_w/msi->ccos_w;
    if (V2 == V_PHOT || V2 == V_ZLL || V2 == V_ZNN) {
      LBq1 = (I3_q-e_q*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
      RBq1 = -e_q*msi->csin_w/msi->ccos_w;
    } else if (V2 == V_WMIN || V2 == V_WPLU) {
      LBq1 = (I3_qp-e_qp*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
      RBq1 = -e_qp*msi->csin_w/msi->ccos_w;
    }
  } else if ((V1 == V_WMIN && e_q>0) || (V1 == V_WPLU && e_q<0)) {
    LAq1 = 0;
    RAq1 = 0;
    LBq1 = 1.0/sqrt(2)/msi->csin_w;
    RBq1 = 0;
  } else if ((V1 == V_WPLU && e_q>0) || (V1 == V_WMIN && e_q<0)) {
    LAq1 = 1.0/sqrt(2)/msi->csin_w;
    RAq1 = 0;
    LBq1 = 0;
    RBq1 = 0;
  }
  if (V2 == V_PHOT) {
    LAq2 = e_q;
    RAq2 = e_q;
    LBq2 = e_q;
    RBq2 = e_q;
  } else if (V2 == V_ZLL || V2 == V_ZNN) {
    LAq2 = (I3_q-e_q*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    RAq2 = -e_q*msi->csin_w/msi->ccos_w;
    LBq2 = (I3_q-e_q*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    RBq2 = -e_q*msi->csin_w/msi->ccos_w;
  } else if (V2 == V_WMIN && (V1 == V_PHOT || V1 == V_ZLL || V1 == V_ZNN)) {
    LAq2 = 1.0/sqrt(2)/msi->csin_w;
    RAq2 = 0;
    LBq2 = 1.0/sqrt(2)/msi->csin_w;
    RBq2 = 0;
  } else if (V2 == V_WPLU && (V1 == V_PHOT || V1 == V_ZLL || V1 == V_ZNN)) {
    LAq2 = 1.0/sqrt(2)/msi->csin_w;
    RAq2 = 0;
    LBq2 = 1.0/sqrt(2)/msi->csin_w;
    RBq2 = 0;
  } else if ((V2 == V_WMIN && e_q>0) || (V2 == V_WPLU && e_q<0)) {
    LAq2 = 1.0/sqrt(2)/msi->csin_w;
    RAq2 = 0;
    LBq2 = 0;
    RBq2 = 0;
  } else if ((V2 == V_WPLU && e_q>0) || (V2 == V_WMIN && e_q<0)) {
    LAq2 = 0;
    RAq2 = 0;
    LBq2 = 1.0/sqrt(2)/msi->csin_w;
    RBq2 = 0;
  } else {
    assert(false);
  }


  N_VV=0.0;
  LZu=(0.5-2.0/3*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
  //  static LZu = osi->msi->cCminus_Zuu;
  LZd=(-0.5+1.0/3*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
  //  static LZd = osi->msi->cCminus_Zdd;
  RZu=-2.0/3*msi->csin_w/msi->ccos_w;
  //  static RZu = osi->msi->cCminus_Zuu;
  RZd=1.0/3*msi->csin_w/msi->ccos_w;
  //  static RZd = osi->msi->cCminus_Zdd;
  LW=1.0/sqrt(2)/msi->csin_w;
  //  static LW = osi->msi->cCminus_W
  if (V1 == V_PHOT && V2 == V_PHOT) {
    N_VV = 2*(2.0/3)*(2.0/3)+(Nf-2)*(1.0/3)*(1.0/3);
  } else if (((V1 == V_ZLL || V1 == V_ZNN) && V2 == V_PHOT) || (V1 == V_PHOT && (V2 == V_ZLL || V2 == V_ZNN))) {
    // note: sign mistake in 1503.04812, consistent with 1112.1531
    N_VV = 0.5*(2.0*(LZu+RZu)*2.0/3.0 + (Nf-2.0)*(LZd+RZd)*(-1.0/3.0));
  } else if ((V1 == V_ZLL || V1 == V_ZNN) && (V2 == V_ZLL || V2 == V_ZNN)) {
    N_VV = 0.5*(2.0*(LZu*LZu+RZu*RZu)+(Nf-2.0)*(LZd*LZd+RZd*RZd));
  } else if ((V1 == V_WMIN || V1 == V_WPLU) && (V2 == V_WMIN || V2 == V_WPLU)) {
    int Ng = Nf/2; // number of massless quark generations
    N_VV = 0.5*Ng*LW*LW;
  }
  

  Q_A[0] = LAq1*LAq2;
  Q_A[1] = RAq1*RAq2;

  Q_B[0] = LBq1*LBq2;
  Q_B[1] = RBq1*RBq2;

  Q_C[0] = N_VV;
  Q_C[1] = N_VV;
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}
