#include "header.hpp"

#include "ppllll04.amplitude.set.hpp"

ppllll04_decay_SR::ppllll04_decay_SR(){
  Logger logger("ppllll04_decay_SR::ppllll04_decay_SR");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  logger << LOG_DEBUG_VERBOSE << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

ppllll04_decay_SR::~ppllll04_decay_SR(){
  Logger logger("ppllll04_decay_SR::~ppllll04_decay_SR");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  logger << LOG_DEBUG_VERBOSE << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


ppllll04_decay_SR::ppllll04_decay_SR(int _resonant_V, TypeV _V1, TypeV _V2, ppllll04_amplitude_set * asi, vector<int> permutation){
  Logger logger("ppllll04_decay_SR::ppllll04_decay_SR");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  resonant_V = _resonant_V;
  V1 = _V1;
  V2 = _V2;
  model_set * msi = asi->msi;
  int q1 = asi->q1;
  
  Q_lep.resize(4);
  I_lep.resize(4);
  
  LR56.resize(2);
  LR78.resize(2);
  
  // couplings of the on-shell VB to the lepton pairs
  LR5_2.resize(2);
  LR7_1.resize(2);
  LR6_2.resize(2);
  LR8_1.resize(2);

  LR_P5.resize(2);
  LR_Z5.resize(2);
  LR_W5.resize(2);
  LR_P6.resize(2);
  LR_Z6.resize(2);
  LR_W6.resize(2);
  LR_P7.resize(2);
  LR_Z7.resize(2);
  LR_W7.resize(2);
  LR_P8.resize(2);
  LR_Z8.resize(2);
  LR_W8.resize(2);


  if (q1==2 || q1==4) { // u, c
    I3_q=0.5;
    //    I3_qp=-0.5;
    e_q=2.0/3;
    e_qp=-2.0/3;
    /*
    I3_q = msi->Iw_u;
    I3_qp = -msi->Iw_u;
    e_q = msi->Q_u;
    e_qp = -msi->Q_u;
    */
  } else if (q1==1 || q1==3 || q1==5) { // d, s, b
    I3_q=-0.5;
    //    I3_qp=0.5;
    e_q=-1.0/3;
    e_qp=1.0/3;
    /*
    I3_q = msi->Iw_d;
    I3_qp = -msi->Iw_d;
    e_q = msi->Q_d;
    e_qp = -msi->Q_d;
    */
  }
  if ( ((V1 == V_PHOT || V1 == V_ZLL) && (V2 == V_PHOT || V2 == V_ZLL)) || (V1 == V_WMIN && V2 == V_WPLU) || (V1 == V_WPLU && V2 == V_WMIN) || (V1 == V_ZLL && V2 == V_ZNN) || (V1 == V_ZNN && V2 == V_ZLL) || (V1 == V_PHOT && V2 == V_ZNN) || (V1 == V_ZNN && V2 == V_PHOT) ) {
    //electrically neutral final state
    e_qp = -e_q;
    //    I3_qp = -I3_q;
  } else if ( ((V1 == V_PHOT || V1 == V_ZLL || V1 == V_ZNN) && V2 == V_WMIN) || (V1 == V_WMIN && (V2 == V_PHOT || V2 == V_ZLL || V2 == V_ZNN)) ) {
    // negatively charged final state
    e_qp = -1-e_q;
    //    I3_qp = I3_q;
  } else if ( ((V1 == V_PHOT || V1 == V_ZLL || V1 == V_ZNN) && V2 == V_WPLU) || (V1 == V_WPLU && (V2 == V_PHOT || V2 == V_ZLL || V2 == V_ZNN)) ) {
    // positively charged final state
    e_qp = 1-e_q;
    //    I3_qp = I3_q;
  } else {
    assert(false);
  }


  if (V1 == V_PHOT || V1 == V_ZLL) {
    Q_lep[0]=-1;
    I_lep[0]=-0.5;
    Q_lep[1]=-1;
    I_lep[1]=-0.5;
  } else if (V1 == V_WMIN) { // FIXIT: something is switched with neutrinos and leptons in W's
    Q_lep[1]=0;
    I_lep[1]=0.5;
    Q_lep[0]=-1;
    I_lep[0]=-0.5;
  } else if (V1 == V_WPLU) {
    Q_lep[1]=-1;
    I_lep[1]=-0.5;
    Q_lep[0]=0;
    I_lep[0]=0.5;
  } else if (V1 == V_ZNN) {
    Q_lep[0]=0;
    I_lep[0]=0.5;
    Q_lep[1]=0;
    I_lep[1]=0.5;
  }
  if (V2 == V_PHOT || V2 == V_ZLL) {
    Q_lep[2]=-1;
    I_lep[2]=-0.5;
    Q_lep[3]=-1;
    I_lep[3]=-0.5;
  } else if (V2 == V_WMIN) { // FIXIT: something is switched with neutrinos and leptons in W's
    Q_lep[3]=0;
    I_lep[3]=0.5;
    Q_lep[2]=-1;
    I_lep[2]=-0.5;
  } else if (V2 == V_WPLU) {
    Q_lep[3]=-1;
    I_lep[3]=-0.5;
    Q_lep[2]=0;
    I_lep[2]=0.5;
  } else if (V2 == V_ZNN) {
    Q_lep[2]=0;
    I_lep[2]=0.5;
    Q_lep[3]=0;
    I_lep[3]=0.5;
  }
  

  if (V1 == V_PHOT) { //photon
    LR56[0] = Q_lep[0];
    LR56[1] = Q_lep[0];

    LR7_1[0] = Q_lep[2]; // FIXIT: something is switched with neutrinos and leptons in W's
    LR7_1[1] = Q_lep[2];
    LR8_1[0] = Q_lep[3];
    LR8_1[1] = Q_lep[3];

    // LR56[0] = -1.0;
    // LR56[1] = -1.0;
    
    // if (V2 == V_PHOT || V2 == V_ZLL) {
    //   LR7_1[0] = -1.0;
    //   LR7_1[1] = -1.0;
    //   LR8_1[0] = -1.0;
    //   LR8_1[1] = -1.0;
    // } else if (V2 == V_ZNN) { // nunugamma->nunull
    //   LR7_1[0] = 0.0;
    //   LR7_1[1] = 0.0;
    //   LR8_1[0] = 0.0;
    //   LR8_1[1] = 0.0;
    // } else if (V2 == V_WMIN) {
    //   LR7_1[0] = -1.0;
    //   LR7_1[1] = -1.0;
    //   LR8_1[0] = 0.0;
    //   LR8_1[1] = 0.0;
    // } else if (V2 == V_WPLU) {
    //   LR7_1[0] = 0.0;
    //   LR7_1[1] = 0.0;
    //   LR8_1[0] = -1.0;
    //   LR8_1[1] = -1.0;
    // }
    /*
    LR56[0] = msi->Q_l;
    LR56[1] = msi->Q_l;
    LR78_1[0] = msi->Q_l;
    LR78_1[1] = msi->Q_l;
    */
  } else if (V1 == V_ZLL || V1 == V_ZNN) { // Z // TODO: depends on the decay!
    // LR56[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    // LR56[1] = msi->csin_w/msi->ccos_w;
    LR56[0] = (I_lep[0]-Q_lep[0]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR56[1] = -Q_lep[0]*msi->csin_w/msi->ccos_w;
    LR7_1[0] = (I_lep[2]-Q_lep[2]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w; // FIXIT: something is switched with neutrinos and leptons in W's
    LR7_1[1] = -Q_lep[2]*msi->csin_w/msi->ccos_w;
    LR8_1[0] = (I_lep[3]-Q_lep[3]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR8_1[1] = -Q_lep[3]*msi->csin_w/msi->ccos_w;
    // if (V2 == V_PHOT || V2 == V_ZLL || V2 == V_ZNN) {
    //   LR7_1[0] = (I_lep[2]-Q_lep[2]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    //   LR7_1[1] = -Q_lep[2]*msi->csin_w/msi->ccos_w;
    //   LR8_1[0] = (I_lep[3]-Q_lep[3]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    //   LR8_1[1] = -Q_lep[3]*msi->csin_w/msi->ccos_w;
    // } else if (V2 == V_WMIN) {
    //   LR7_1[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    //   LR7_1[1] = msi->csin_w/msi->ccos_w;
    //   LR8_1[0] = (+0.5)/msi->csin_w/msi->ccos_w;
    //   LR8_1[1] = 0.0;
    // } else if (V2 == V_WPLU) {
    //   LR7_1[0] = (+0.5)/msi->csin_w/msi->ccos_w;
    //   LR7_1[1] = 0.0;
    //   LR8_1[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    //   LR8_1[1] = msi->csin_w/msi->ccos_w;
    // }
    /*
    LR56[0] = msi->cCminus_Zee;
    LR56[1] = msi->cCplus_Zee;
    LR78_1[0] = msi->cCminus_Zee;
    LR78_1[1] = msi->cCplus_Zee;
    // Z -> nn~ decay !!!
    LR56[0] = msi->cCminus_Znn;
    LR56[1] = msi->cCplus_Znn;
    LR78_1[0] = msi->cCminus_Znn;
    LR78_1[1] = msi->cCplus_Znn;
    */
  } else if (V1 == V_WMIN || V1 == V_WPLU) { // W
    LR56[0] = 1.0/sqrt(2)/msi->csin_w;
    LR56[1] = 0.0;
    LR7_1[0] = 1.0/sqrt(2)/msi->csin_w;
    LR7_1[1] = 0.0;
    LR8_1[0] = 1.0/sqrt(2)/msi->csin_w;
    LR8_1[1] = 0.0;
    /*
    LR56[0] = msi->cCminus_W;
    LR56[1] = 0.;
    LR78_1[0] = msi->cCminus_W;
    LR78_1[1] = 0.;
    */
  }
  if (V2 == V_PHOT) { //photon
    LR78[0] = -1.0;
    LR78[1] = -1.0;
    if (V1 == V_PHOT || V1 == V_ZLL) {
      LR5_2[0] = -1.0;
      LR5_2[1] = -1.0;
      LR6_2[0] = -1.0;
      LR6_2[1] = -1.0;
    } else if (V1 == V_ZNN) { // nunugamma->nunull
      LR5_2[0] = 0.0;
      LR5_2[1] = 0.0;
      LR6_2[0] = 0.0;
      LR6_2[1] = 0.0;
    } else if (V1 == V_WMIN) {
      assert(false);
    } else if (V1 == V_WPLU) {
      assert(false);
    }
    /*
    LR78[0] = msi->Q_l;
    LR78[1] = msi->Q_l;
    LR56_2[0] = msi->Q_l;
    LR56_2[1] = msi->Q_l;
    */
  } else if (V2 == V_ZLL || V2 == V_ZNN) { // Z
    LR78[0] = (I_lep[2]-Q_lep[2]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR78[1] = -Q_lep[2]*msi->csin_w/msi->ccos_w;
    LR5_2[0] = (I_lep[0]-Q_lep[0]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR5_2[1] = -Q_lep[0]*msi->csin_w/msi->ccos_w;
    LR6_2[0] = (I_lep[1]-Q_lep[1]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR6_2[1] = -Q_lep[1]*msi->csin_w/msi->ccos_w;
    /*
    LR78[0] = msi->cCminus_Zee;
    LR78[1] = msi->cCplus_Zee;
    LR56_2[0] = msi->cCminus_Zee;
    LR56_2[1] = msi->cCplus_Zee;
    // Z -> nn~ decay !!!
    LR78[0] = msi->cCminus_Znn;
    LR78[1] = msi->cCplus_Znn;
    LR56_2[0] = msi->cCminus_Znn;
    LR56_2[1] = msi->cCplus_Znn;
    */
  } else if (V2 == V_WMIN || V2 == V_WPLU) { // W
    LR78[0] = 1.0/sqrt(2)/msi->csin_w;
    LR78[1] = 0.0;
    if (V1 == V_WMIN || V1 == V_WPLU) {
      // WW
      LR5_2[0] = 1.0/sqrt(2)/msi->csin_w;
      LR5_2[1] = 0.0;
      LR6_2[0] = 1.0/sqrt(2)/msi->csin_w;
      LR6_2[1] = 0.0;
    }
    else if (V1 == V_PHOT || V1 == V_ZLL){
      if (V2 == V_WMIN) {
      // Z/gamma W^- --> ll W^-
      // DY-like boson is a W^-
      // the resonant W^- can only couple to the iatnineutrino, not the lepton
	LR6_2[0] = 1.0/sqrt(2)/msi->csin_w;
	LR6_2[1] = 0.0;
	LR5_2[0] = 0.0;
	LR5_2[1] = 0.0;
      }
      else if (V2 == V_WPLU) {
      // Z/gamma W^+ --> ll W^+
      // DY-like boson is a W^+
      // the resonant W^+ can only couple to the neutrino, not the antilepton
	LR5_2[0] = 1.0/sqrt(2)/msi->csin_w;
	LR5_2[1] = 0.0;
	LR6_2[0] = 0.0;
	LR6_2[1] = 0.0;
      }
    }
    else if (V1 == V_ZNN){
      if (V2 == V_WMIN) {
      // Z W^- ->nunu W^-
      // DY-like boson is a W^-
      // the resonant W^- can only couple to the iatnineutrino, not the lepton
	LR6_2[0] = 0.0;
	LR6_2[1] = 0.0;
	LR5_2[0] = 1.0/sqrt(2)/msi->csin_w;
	LR5_2[1] = 0.0;
      }
      else if (V2 == V_WPLU) {
      // Z W^+ ->nunu W^+
      // DY-like boson is a W^+
      // the resonant W^+ can only couple to the neutrino, not the antilepton
	LR5_2[0] = 0.0;
	LR5_2[1] = 0.0;
	LR6_2[0] = 1.0/sqrt(2)/msi->csin_w;
	LR6_2[1] = 0.0;
      }
    }
    else {
      assert(false);
    }
  }


  L_P=0.0;
  L_Z=0.0;
  L_W=0.0;
  R_P=0.0;
  R_Z=0.0;
  R_W=0.0;
  
  // quark boson couplings
  if (e_q*e_qp>0) { // udb or dub
    L_W=1.0/sqrt(2)/msi->csin_w;
    R_W=0.0;
    /*
    L_W = msi->cCminus_W;
    R_W = 0.;
    */
  } else { // uub or ddb
    L_P=e_q;
    R_P=e_q;
    L_Z=(I3_q-e_q*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    R_Z=-e_q*msi->csin_w/msi->ccos_w;
  }
  

  // for the second VB becoming resonant
  if (V1 == V_PHOT || V1 == V_ZLL || V1 == V_ZNN) { // LR_W6 is probably LR_W5 and so on...   SOLUTION: THE IDEA IS THAT THE NUMBER CORRESPONDS TO WHERE THE NON-DY BOSON COUPLES
    LR_P5[0] = Q_lep[1]; // !!!!! 5<->6 is switched !!!!!! so we switched 1<->0 !!!! :-) !!!!    SOLUTION: THE IDEA IS THAT THE NUMBER CORRESPONDS TO WHERE THE NON-DY BOSON COUPLES
    LR_P5[1] = Q_lep[1];
    LR_P6[0] = Q_lep[0];
    LR_P6[1] = Q_lep[0];
    LR_Z5[0] = (I_lep[1]-Q_lep[1]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR_Z5[1] = -Q_lep[1]*msi->csin_w/msi->ccos_w;
    LR_Z6[0] = (I_lep[0]-Q_lep[0]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR_Z6[1] = -Q_lep[0]*msi->csin_w/msi->ccos_w;
    if (V2 == V_WMIN) {
      LR_W5[0] = 1.0/sqrt(2)/msi->csin_w; // should be zero! But no problem, as contribution already forced zero above
      LR_W5[1] = 0.0;
      LR_W6[0] = 1.0/sqrt(2)/msi->csin_w;
      LR_W6[1] = 0.0;
    } else if (V2 == V_WPLU) {
      LR_W5[0] = 1.0/sqrt(2)/msi->csin_w;
      LR_W5[1] = 0.0;
      LR_W6[0] = 1.0/sqrt(2)/msi->csin_w; // should be zero! But no problem, as contribution already forced zero above
      LR_W6[1] = 0.0;
    } else {
      LR_W5[0] = 0.0;
      LR_W5[1] = 0.0;
      LR_W6[0] = 0.0;
      LR_W6[1] = 0.0;
    }
    /*
    LR_P5[0] = msi->Q_l;
    LR_P5[1] = msi->Q_l;
    LR_P6[0] = msi->Q_l;
    LR_P6[1] = msi->Q_l;
    LR_Z5[0] = msi->cCminus_Zee;
    LR_Z5[1] = msi->cCplus_Zee;
    LR_Z6[0] = msi->cCminus_Zee;
    LR_Z6[1] = msi->cCplus_Zee;
    LR_W5[0] = 0.0;
    LR_W5[1] = 0.0;
    LR_W6[0] = 0.0;
    LR_W6[1] = 0.0;
    // Z -> nn~ decay !!!
    LR_P5[0] = msi->Q_n;
    LR_P5[1] = msi->Q_n;
    LR_P6[0] = msi->Q_n;
    LR_P6[1] = msi->Q_n;
    LR_Z5[0] = msi->cCminus_Znn;
    LR_Z5[1] = msi->cCplus_Znn;
    LR_Z6[0] = msi->cCminus_Znn;
    LR_Z6[1] = msi->cCplus_Znn;
    LR_W5[0] = 0.0;
    LR_W5[1] = 0.0;
    LR_W6[0] = 0.0;
    LR_W6[1] = 0.0;
    */
  } else if (V1 == V_WMIN) { // V1=W^- ==> 5=l^-, 6=nubar;  gamma/Z -> l^-nu W^+ -> l^-nu l^+nu  or  W^- -> l^-nu gamma/Z -> l^-nu l^-l^+
    // LR_P6[0] = -1.0; // NOTE 6<->5 switched; SOLUTION: THE IDEA IS THAT THE NUMBER CORRESPONDS TO WHERE THE NON-DY BOSON COUPLES
    // LR_P6[1] = -1.0;
    // LR_P5[0] = 0.0;  // Photon does not couple to nu ==> LR_P=0 for 6=nubar ==> LR_P5 = 0
    // LR_P5[1] = 0.0;
    // LR_Z6[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    // LR_Z6[1] = msi->csin_w/msi->ccos_w;
    // LR_Z5[0] = (+0.5)/msi->csin_w/msi->ccos_w;
    // LR_Z5[1] = 0.0;
    // LR_W5[0] = 1.0/sqrt(2)/msi->csin_w;
    // LR_W5[1] = 0.0;
    // LR_W6[0] = 1.0/sqrt(2)/msi->csin_w;
    // LR_W6[1] = 0.0;

    LR_P5[0] = Q_lep[1]; // NOTE 6<->5 switched; SOLUTION: THE IDEA IS THAT THE NUMBER CORRESPONDS TO WHERE THE NON-DY BOSON COUPLES; thus we switche 1<->0 as well
    LR_P5[1] = Q_lep[1];
    LR_P6[0] = Q_lep[0];
    LR_P6[1] = Q_lep[0];
    LR_Z5[0] = (I_lep[1]-Q_lep[1]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR_Z5[1] = -Q_lep[1]*msi->csin_w/msi->ccos_w;
    LR_Z6[0] = (I_lep[0]-Q_lep[0]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR_Z6[1] = -Q_lep[0]*msi->csin_w/msi->ccos_w;
    LR_W5[0] = 1.0/sqrt(2)/msi->csin_w;
    LR_W5[1] = 0.0;
    LR_W6[0] = 1.0/sqrt(2)/msi->csin_w;
    LR_W6[1] = 0.0;
    

    /*
    LR_P5[0] = msi->Q_l;
    LR_P5[1] = msi->Q_l;
    LR_P6[0] = msi->Q_n;
    LR_P6[1] = msi->Q_n;
    LR_Z5[0] = msi->cCminus_Zee;
    LR_Z5[1] = msi->cCplus_Zee;
    LR_Z6[0] = msi->cCminus_Znn;
    LR_Z6[1] = msi->cCplus_Znn;
    LR_W5[0] = msi->cCminus_W;
    LR_W5[1] = 0.0; // msi->cCplus_W;
    LR_W6[0] = msi->cCminus_W;
    LR_W6[1] = 0.0; // msi->cCplus_W;
    */
  } else {
    assert(false);
  }
  if (V2 == V_PHOT || V2 == V_ZLL || V2 == V_ZNN) {
    // LR_P8[0] = -1.0; // !!!!! 7<->8 is switched !!!!!! so we switched 2<->3 !!!! :-) !!!!    SOLUTION: THE IDEA IS THAT THE NUMBER CORRESPONDS TO WHERE THE NON-DY BOSON COUPLES
    // LR_P8[1] = -1.0;
    // LR_P7[0] = -1.0;
    // LR_P7[1] = -1.0;
    // LR_Z8[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    // LR_Z8[1] = msi->csin_w/msi->ccos_w;
    // LR_Z7[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    // LR_Z7[1] = msi->csin_w/msi->ccos_w;

    // if (V1 == V_WMIN) {
    //   // FIXME: Z->nunu!
    //   LR_W8[0] = 1.0/sqrt(2)/msi->csin_w;
    //   LR_W8[1] = 0.0;
    //   LR_W7[0] = 1.0/sqrt(2)/msi->csin_w;
    //   LR_W7[1] = 0.0;
    // } else if (V2 == V_WPLU) {
    //   LR_W8[0] = 1.0/sqrt(2)/msi->csin_w;
    //   LR_W8[1] = 0.0;
    //   LR_W7[0] = 0.0;
    //   LR_W7[1] = 0.0;
    // } else {
    //   LR_W8[0] = 0.0;
    //   LR_W8[1] = 0.0;
    //   LR_W7[0] = 0.0;
    //   LR_W7[1] = 0.0;
    // }


    LR_P7[0] = Q_lep[3]; // !!!!! 7<->8 is switched !!!!!! so we switched 2<->3 !!!! :-) !!!!    SOLUTION: THE IDEA IS THAT THE NUMBER CORRESPONDS TO WHERE THE NON-DY BOSON COUPLES
    LR_P7[1] = Q_lep[3];
    LR_P8[0] = Q_lep[2];
    LR_P8[1] = Q_lep[2];
    LR_Z7[0] = (I_lep[3]-Q_lep[3]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR_Z7[1] = -Q_lep[3]*msi->csin_w/msi->ccos_w;
    LR_Z8[0] = (I_lep[2]-Q_lep[2]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR_Z8[1] = -Q_lep[2]*msi->csin_w/msi->ccos_w;
    if (V2 == V_WPLU) {
      LR_W7[0] = 1.0/sqrt(2)/msi->csin_w;
      LR_W7[1] = 0.0;
      LR_W8[0] = 1.0/sqrt(2)/msi->csin_w; // should be zero! But no problem, as contribution already forced zero above
      LR_W8[1] = 0.0;
    } else if (V2 == V_WMIN) {
      LR_W7[0] = 0.0;
      LR_W7[1] = 0.0;
      LR_W8[0] = 1.0/sqrt(2)/msi->csin_w;
      LR_W8[1] = 0.0;
    } else {
      LR_W7[0] = 0.0;
      LR_W7[1] = 0.0;
      LR_W8[0] = 0.0;
      LR_W8[1] = 0.0;
    }

    /*
    LR_P8[0] = msi->Q_l;
    LR_P8[1] = msi->Q_l;
    LR_P7[0] = msi->Q_l;
    LR_P7[1] = msi->Q_l;
    LR_Z8[0] = msi->cCminus_Zee;
    LR_Z8[1] = msi->cCplus_Zee;
    LR_Z7[0] = msi->cCminus_Zee;
    LR_Z7[1] = msi->cCplus_Zee;
    LR_W8[0] = 0.0;
    LR_W8[1] = 0.0;
    LR_W7[0] = 0.0;
    LR_W7[1] = 0.0;
    // Z -> nn~ decay !!!
    LR_P8[0] = msi->Q_n;
    LR_P8[1] = msi->Q_n;
    LR_P7[0] = msi->Q_n;
    LR_P7[1] = msi->Q_n;
    LR_Z8[0] = msi->cCminus_Znn;
    LR_Z8[1] = msi->cCplus_Znn;
    LR_Z7[0] = msi->cCminus_Znn;
    LR_Z7[1] = msi->cCplus_Znn;
    LR_W8[0] = 0.0;
    LR_W8[1] = 0.0;
    LR_W7[0] = 0.0;
    LR_W7[1] = 0.0;
    */
  } else if (V2 == V_WMIN || V2 == V_WPLU) {
    // relevant for W^+W^-
    // LR_P8[0] = -1.0;
    // LR_P8[1] = -1.0;
    // LR_P7[0] = 0.0;
    // LR_P7[1] = 0.0;
    // LR_Z8[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    // LR_Z8[1] = msi->csin_w/msi->ccos_w;
    // LR_Z7[0] = (+0.5)/msi->csin_w/msi->ccos_w;
    // LR_Z7[1] = 0.0;
    // LR_W8[0] = 1.0/sqrt(2)/msi->csin_w;
    // LR_W8[1] = 0.0;
    // LR_W7[0] = 1.0/sqrt(2)/msi->csin_w;
    // LR_W7[1] = 0.0;
    //  } else if (V2 == V_WPLU) {
    // relevant for W^-W^+
    // LR_P7[0] = -1.0;
    // LR_P7[1] = -1.0;
    // LR_P8[0] = 0.0;
    // LR_P8[1] = 0.0;
    // LR_Z7[0] = (-0.5+msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    // LR_Z7[1] = msi->csin_w/msi->ccos_w;
    // LR_Z8[0] = (+0.5)/msi->csin_w/msi->ccos_w;
    // LR_Z8[1] = 0.0;
    // // relevant for ZW
    // LR_W8[0] = 1.0/sqrt(2)/msi->csin_w;
    // LR_W8[1] = 0.0;
    // LR_W7[0] = 1.0/sqrt(2)/msi->csin_w;
    // LR_W7[1] = 0.0;
    // }

    LR_P7[0] = Q_lep[3]; // NOTE 7<->8 switched; SOLUTION: THE IDEA IS THAT THE NUMBER CORRESPONDS TO WHERE THE NON-DY BOSON COUPLES; thus we switche 3<->2 as well
    LR_P7[1] = Q_lep[3];
    LR_P8[0] = Q_lep[2];
    LR_P8[1] = Q_lep[2];
    LR_Z7[0] = (I_lep[3]-Q_lep[3]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR_Z7[1] = -Q_lep[3]*msi->csin_w/msi->ccos_w;
    LR_Z8[0] = (I_lep[2]-Q_lep[2]*msi->csin_w*msi->csin_w)/msi->csin_w/msi->ccos_w;
    LR_Z8[1] = -Q_lep[2]*msi->csin_w/msi->ccos_w;
    // relevant for ZW
    LR_W7[0] = 1.0/sqrt(2)/msi->csin_w;
    LR_W7[1] = 0.0;
    LR_W8[0] = 1.0/sqrt(2)/msi->csin_w;
    LR_W8[1] = 0.0;
    
    /*
    LR_P8[0] = msi->Q_l;
    LR_P8[1] = msi->Q_l;
    LR_P7[0] = msi->Q_n;
    LR_P7[1] = msi->Q_n;
    LR_Z8[0] = msi->cCminus_Zee;
    LR_Z8[1] = msi->cCplus_Zee;
    LR_Z7[0] = msi->cCminus_Znn;
    LR_Z7[1] = msi->cCplus_Znn;
    LR_W8[0] = msi->cCminus_W;
    LR_W8[1] = 0.0; // msi->cCplus_W;
    LR_W7[0] = msi->cCminus_W;
    LR_W7[1] = 0.0; // msi->cCplus_W;
    */
  } else {
    assert(false);
  }
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}
