#ifndef EVENTSET_H
#define EVENTSET_H

#include <map>
#include <string>
#include <vector>

using namespace std;

#include "user.defined.hpp"
#include "qTsubtraction.basic.hpp"

class inputparameter_set;
class model_set;
class contribution_set;
class phasespace_set;
class fiducialcut;

class event_set{
private:

public:
  event_set();
  virtual ~event_set();

  void phasespacepoint();
  void define_fiducial_cut_list();
  void determine_p_parton();
  void determine_n_partonlevel();
  void advanced_sort_by_pT(int x_a, int j_g);

  virtual void particles(int i_a){}
  virtual void cuts(int i_a){}
  virtual void phasespacepoint_born(){}
  virtual void phasespacepoint_collinear(){}
  virtual void phasespacepoint_real(){}
  virtual void phasespacepoint_realcollinear(){}
  virtual void phasespacepoint_doublereal(){}

  //  event/event.initialization.cpp
  void initialization(inputparameter_set * isi, contribution_set * _csi, model_set * _msi, user_defined * _user);
  void initialization_set_default();
  void initialization_input(inputparameter_set * isi);
  void initialization_after_input();

  //  event/event.initialization.run.cpp
  void initialization_complete();
  void initialization_integration();
  void check_parton_content();

  //  event/event.initialization.object.list.cpp
  void fill_object_list_basic();
  void fill_object_list_user();
  void determination_equivalent_object_lep_lm_lp(string lep, string lm, string lp);
  void determination_equivalent_object_lep_e_mu_tau(string lep, string e, string mu, string tau);
  void determination_equivalent_object();
  void determination_object_list_selection();

  //  event/event.initialization.object.group.cpp
  void determination_index_group_parton();
  void determination_index_group();
  void determination_runtime_object();

  //  event/event.initialization.object.definition.cpp
  void determination_object_definition();
  void determination_define_pT_lep_lm_lp(string lep, string lm, string lp);
  void determination_define_pT_lep_e_mu_tau(string lep, string e, string mu, string tau);
  void determination_define_ET_lep_lm_lp(string lep, string lm, string lp);
  void determination_define_ET_lep_e_mu_tau(string lep, string e, string mu, string tau);
  void determination_define_eta_lep_lm_lp(string lep, string lm, string lp);
  void determination_define_eta_lep_e_mu_tau(string lep, string e, string mu, string tau);
  void determination_define_y_lep_lm_lp(string lep, string lm, string lp);
  void determination_define_y_lep_e_mu_tau(string lep, string e, string mu, string tau);
  void determination_n_observed_min_lep_lm_lp(string lep, string lm, string lp);
  void determination_n_observed_min_lep_e_mu_tau(string lep, string e, string mu, string tau);
  void determination_n_observed_max_lep_lm_lp(string lep, string lm, string lp);
  void determination_n_observed_max_lep_e_mu_tau(string lep, string e, string mu, string tau);

  //  event/event.initialization.object.partonlevel.cpp
  void determination_object_partonlevel_phasespace();
  void determination_object_observed_from_partonlevel();
  void determination_n_observed_max_from_partonlevel_lep_lm_lp(string lep, string lm, string lp);
  void determination_n_observed_min_from_partonlevel_lep_lm_lp(string lep, string lm, string lp);
  void determination_n_observed_max_from_partonlevel_lep_e_mu_tau(string lep, string e, string mu, string tau);
  void determination_n_observed_min_from_partonlevel_lep_e_mu_tau(string lep, string e, string mu, string tau);

  //  event/event.output.cpp
  string output_object_list_no_access();
  string output_object_list();
  string output_object_list_selection();
  string output_equivalent_object();
  string output_equivalent_inverse_object();
  string output_index_group_all_parton();
  string output_index_group_original();
  string output_index_group_jet();
  string output_particle_definition();
  string output_particle_definition_selection();
  string output_list_fiducialcut();
  string output_index_group_parton();
  void output_runtime_object();
  void output_phasespacepoints_check_selection();

  //  event/event.selection.cpp
  void perform_event_selection();
  void perform_event_selection_phasespace();
  void perform_event_selection_user_defined_particles_phasespace();
  void convert_event_format_phasespace(int begin, int end);

  //  event/event.selection.output.cpp
  void output_parton_momenta_LAB();
  void output_photon_recombination();
  void output_runtime_original(size_t i_g, size_t j_g);
  void output_runtime_original_summary();
  void output_runtime_missing();
  void output_runtime_missing_summary();
  void output_protojet_determination();
  void output_frixione_isolation();
  void output_frixione_isolation_multiphoton(bool flag_recombination_photon_photon);
  void output_isolated_photon();
  void output_frixione_isolation_summary();
  void output_photon_jet_algorithm();
  void output_protojet_removal_beam(bool check_removal);
  void output_jet_algorithm_protojet();
  void output_jet_algorithm_jet();
  void output_frixione_jet_removal();
  void output_summary_frixione_jet_removal(bool check_removal);
  void output_jet_interpretation();
  void output_jet_algorithm_result(size_t i_g, size_t j_g);
  void output_jet_algorithm_result_summary();

  // event/event.photon.recombination.cpp
  void perform_photon_recombination();
  bool perform_photon_photon_recombination();

  // event/event.photon.isolation.cpp
  double frixione_discr(double delta);
  double frixione_discr(double delta, double delta_dynamic);
  double frixione_discriminant_R2(double delta);
  double frixione_discriminant_R2(double delta, double delta_dynamic);
  bool perform_frixione_isolation(particle & photon);

  // event/event.jet.algorithm.cpp
  void jet_algorithm_flavour();
  void sc_Ellis_Soper_flavour();
  void sc_Ellis_Soper_mod_flavour();
  void kTrun2_flavour();
  void antikT_flavour();

  //  event/event.recoil.qT.cpp
  void perform_QT_boost_cms();
  void perform_QT_boost_recoil();

  
  vector<string> object_list;
  vector<string> object_list_selection;
  vector<define_particle_set> pda;  // particle definition all
  vector<define_particle_set> pds;  // particle definition selection

  vector<string> name_fiducial_cut;
  vector<string> fiducial_cut_list;
  vector<fiducialcut> fiducial_cut;


  map<string, int> observed_object;
  map<string, int> access_object;
  map<string, int> access_object_selection;
  map<string, string> equivalent_object;
  map<int, int> equivalent_index;
  map<string, vector<string> > equivalent_inverse_object;
  map<int, vector<size_t> > equivalent_inverse_index;

  vector<vector<vector<int> > > index_group_all_parton;
  vector<int> index_group_original;
  vector<int> index_group_jet;
  
  vector<vector<int> > index_group_protojet_parton;
  vector<vector<int> > index_group_photon_parton;
  vector<vector<int> > index_group_recombination_parton;
  vector<vector<int> > index_group_missing_parton;
  vector<vector<int> > ps_n_partonlevel;

  // different usage (phase space optimization):
  vector<vector<int> > ps_runtime_subtraction_QCD;
  vector<vector<int> > ps_runtime_subtraction_QEW;

  // Could be useful for debug output (not used elsewhere):
  vector<vector<int> > runtime_order;
  vector<vector<int> > runtime_order_inverse;


  
  int frixione_isolation;
  double frixione_n;
  double frixione_epsilon;
  double frixione_fixed_ET_max;
  double frixione_delta_0;
  int frixione_jet_removal;

  vector<string> photon_recombination_selection;
  vector<string> photon_recombination_disable;
  vector<int> photon_recombination_list;

  int photon_recombination;
  int photon_R_definition;
  double photon_R;
  double photon_R2;
  double photon_E_threshold_ratio;
  int photon_jet_algorithm;

  int photon_photon_recombination;
  double photon_photon_recombination_R;

  vector<string> jet_algorithm_selection;
  vector<string> jet_algorithm_disable;
  vector<int> jet_algorithm_list;

  int jet_algorithm;
  int jet_R_definition;
  double jet_R;
  double jet_R2;
  double parton_y_max;
  double parton_eta_max;


  vector<vector<fourvector> > p_parton;
  vector<vector<fourvector> > start_p_parton;

  vector<vector<vector<particle> > > user_particle;

  vector<vector<int> > n_object;
  vector<vector<vector<vector<particle> > > > particle_event;


  int ix_y;
  int ix_a;
  
  size_t j_g_missing;
  size_t j_g_photon;
  size_t j_g_jet;

  // 1st: direction
  // 2nd: phase space index
  vector<vector<int> > event_cut_ps;
  // event_cut_ps not used at present -> direction-dependent cuts
  vector<vector<vector<particle> > > event_original;
  vector<vector<vector<particle> > > event_photon;
  vector<vector<vector<particle_history> > > event_jet;
  vector<vector<particle> > event_missing;
  // 3rd: index of particle group (so far in object list)
  // 4th: pT-ordered pointers to respective pT-ordered particles
  vector<vector<vector<vector<particle *> > > > event_particle_group;

  // temporary entries: 
  vector<particle_history> temp_protojet;
  // used in event selection: could be treated in more elegant way...
  // use vector of pointers to photon particles ???
  vector<int> index_unrecombined_photon;
  vector<int> index_unisolated_photon;


  
  vector<int> cut_ps;
  int first_non_cut_ps;
  vector<int> change_cut;

  int switch_output_cutinfo;
  int switch_output_check_selection;
  stringstream info_cut;

  model_set * msi;
  contribution_set * csi;
  phasespace_set * psi;
  user_defined * user;

  qTsubtraction_basic * qT_basic;
};
#endif
