#ifndef PHASESPACESET_H
#define PHASESPACESET_H

#include <string>
#include <vector>

#include "randommanager.hpp"
#include "logger.hpp"
#include "multichannel.set.hpp"
#include "importancesampling.set.hpp"

using namespace std;

class phasespace_set{
private:

public:
  phasespace_set();
  virtual ~phasespace_set();

  void ax_psp(int x_a);
  void ac_psp(int x_a, int channel);
  void ag_psp(int x_a, int zero);

  void ac_tau_psp(int x_a, vector<int> & tau_MC_map);
  void ac_tau_psp_dipole(int x_a, vector<int> & tau_MC_map);
  void ac_tau_psp_doubledipole(int x_a, vector<int> & tau_MC_map);

  void ax_psp_dipole(int x_a);
  void ac_psp_dipole(int x_a, int channel);
  void ag_psp_dipole(int x_a, int zero);

  void optimize_minv();
  int determination_MCchannels(int x_a);
  int determination_MCchannels_dipole(int x_a);

  void calculate_g_tot();
  void calculate_IS();
  void calculate_IS_RA();
  void output_check_tau_0();
  void set_xz_coll_pdf();
  void handling_cut_psp();
  void handling_techcut_psp();

  
  virtual void optimize_minv_born(){}
  virtual int determination_MCchannels_born(int x_a){return 0;}
  virtual void ac_tau_psp_born(int x_a, vector<int> & tau_MC_map){}
  virtual void ax_psp_born(int x_a){}
  virtual void ac_psp_born(int x_a, int channel){}
  virtual void ag_psp_born(int x_a, int zero){}

  virtual void optimize_minv_real(){}
  virtual int determination_MCchannels_real(int x_a){return 0;}
  virtual void ac_tau_psp_real(int x_a, vector<int> & tau_MC_map){}
  virtual void ax_psp_real(int x_a){}
  virtual void ac_psp_real(int x_a, int channel){}
  virtual void ag_psp_real(int x_a, int zero){}

  virtual void optimize_minv_doublereal(){}
  virtual int determination_MCchannels_doublereal(int x_a){return 0;}
  virtual void ac_tau_psp_doublereal(int x_a, vector<int> & tau_MC_map){}
  virtual void ax_psp_doublereal(int x_a){}
  virtual void ac_psp_doublereal(int x_a, int channel){}
  virtual void ag_psp_doublereal(int x_a, int zero){}

  
  // phasespace.recoil.qT.cpp
  void calculate_IS_QT_phi();
  void calculate_IS_QT();

  double c_costheta_pTcut(double & r1, double & costheta_pTcut, double n);
  double g_costheta_pTcut(double & costheta, double & costheta_pTcut, double n);
  double inv_costheta_pTcut(double & costheta, double & costheta_pTcut, double n);

  // phasespace.initialization.cpp
  void initialization(inputparameter_set * isi, contribution_set * _csi, model_set * _msi, event_set * _esi, user_defined * _user);
  void initialization_set_default();
  void initialization_input(inputparameter_set * isi);
  void initialization_after_input();

  // phasespace.initialization.run.cpp
  // Shift completely to csi ??? Not completely, but remainder should be moved elsewhere !!!
  //  void initialization_contribution_order();
  void initialization_complete();
  void initialization_phasespace_optimization_fiducialcut();
  // Re-organize the connection of dipole !!!
  void initialization_complete_RA();
  void initialization_optimization();
  void initialization_optimization_grid();
  void initialization_filename();
  // It makes sense to have an independent set of masses and widths only for phase space generation !!!
  void initialization_masses();
  // Most of these functions don't really belong here -> should be grouped in a more meaningful way !!!
  void initialization_MC_IS_name_type();
  void initialization_MC_IS_name_list();

  void initialization_phasespace_born();
  void initialization_phasespace_RA();

  void initialization_phasespace_subprocess();
  void initialization_phasespace_subprocess_optimization();
  void initialization_phasespace_subprocess_dipole_IS_RA();
  void initialization_phasespace_subprocess_dipole_RA();
  void initialization_phasespace_subprocess_optimization_dipole_RA();

  void initialization_phasespace_MC_tau();
  void initialization_phasespace_MC_x_dipole_RA();

  void initialization_phasespace_IS();
  void initialization_phasespace_IS_QT();

  void initialization_MC_rng();
  void initialization_MC();

  void initialization_fake_dipole_mapping_RT();
  void initialization_fake_dipole_mapping_RRA();

  //  phasespace/phasespace.optimization.cpp
  void psp_optimization_complete(double & integrand, double & this_psp_weight, double & this_psp_weight2, double & optimization_modifier);
  void step_optimization_complete();
  void result_optimization_complete();
  void output_optimization_complete();
  void enforce_end_optimization();

  // phasespace.initialization.initial.cpp
  void initialization_minimum_tau();
  void initialization_minimum_phasespacecut();

  //  phasespace/phasespace.mapping.initial.cpp
  void calculate_initial_tau_IS_x1x2_IS();
  void calculate_initial_tau_IS_x1x2();
  void calculate_initial_tau_x1x2_IS();
  void calculate_initial_tau_x1x2();
  void calculate_initial_tau_fixed_x1x2_IS();
  void calculate_initial_tau_fixed_x1x2();
  void calculate_initial_collinear_z1z2_IS();

  //  phasespace/phasespace.mapping.cpp
  double c_phi(double r1);
  double inv_phi(double & phi);

  double c_costheta(double & r1);
  double inv_costheta(double & costheta);

  void c_propagator(int i_a, int m, int out, double & smin, double & smax, double r1);
  double g_propagator(int i_a, int m, int out, double smin, double smax);
  void inv_propagator(int i_a, int m, int out, double smin, double smax, double & r1);

  double c_propagator_Breit_Wigner(double & r1, int m, double & smin, double & smax);
  double g_propagator_Breit_Wigner(double & s, int m, double & smin, double & smax);
  double inv_propagator_Breit_Wigner(double s, int m, double & smin, double & smax);

  double c_propagator_narrow_width();
  double g_propagator_narrow_width(int m);
  double inv_propagator_narrow_width();

  double c_propagator_vanishing_width(double & r, double & mass2, double & smin, double & smax, double & nu);
  double g_propagator_vanishing_width(double s, double mass2, double smin, double smax, double nu);
  double inv_propagator_vanishing_width(double s, double mass2, double smin, double smax, double nu);

  void c_timelikeinvariant(int i_a, int out, double & smin, double & smax, double r1);
  double g_timelikeinvariant(double smin, double smax);
  void inv_timelikeinvariant(int i_a, int out, double smin, double smax, double & r1);

  double c_t_propagator(double & r1, int m, double smin, double smax);
  double g_t_propagator(double s, int m, double smin, double smax);
  double inv_t_propagator(double s, int m, double smin, double smax);

  void c_decay(int i_a, int in, int out1, int out2, double r1, double r2);
  double g_decay(int i_a, int in, int out1, int out2);
  void inv_decay(int i_a, int in, int out1, int out2, double & r1, double & r2);

  void c_tchannel(int i_a, int m, int in, int in1, int in2, int out1, int out2, double r1, double r2);
  double g_tchannel(int i_a, int m, int in, int in1, int in2, int out1, int out2);
  void inv_tchannel(int i_a, int m, int in, int in1, int in2, int out1, int out2, double & r1, double & r2);

  void c_tchannel_opt(int i_a, int m, int out, int in1, int in2, int out1, int out2, double r1, double r2);
  double g_tchannel_opt(int i_a, int m, int out, int in1, int in2, int out1, int out2);
  void inv_tchannel_opt(int i_a, int m, int out, int in1, int in2, int out1, int out2, double & r1, double & r2);

  ////////////////////////////////////////

  
  //  phasespace/phasespace.correction.cpp
  void correct_phasespacepoint_real();
  void correct_phasespacepoint_dipole();


  //  dipolesubtraction/phasespace.determination.dipoles.cpp
  void determine_dipole_phasespace_RA();
  void phasespace_ij_k(int x_a);
  void phasespace_ij_a(int x_a);
  void phasespace_ai_k(int x_a);
  void phasespace_ai_b(int x_a);
  void phasespace_ij_k_massive(int x_a);
  void phasespace_ij_a_massive(int x_a);

  //  dipolesubtraction/phasespace.generation.dipoles.cpp
  void ac_psp_RA_dipoles();
  void ag_psp_RA_dipoles();

  //  dipolesubtraction/phasespace.generation.dipoles.CS.cpp
  void ac_psp_RA_group_ij_k(int x_a);
  void ag_psp_RA_group_ij_k(int x_a);
  void ac_psp_RA_group_ij_a(double & sinx_ij_a_min, int x_a);
  void ag_psp_RA_group_ij_a(double & sinx_ij_a_min, int x_a);
  void ac_psp_RA_group_ai_k(double & sinx_ik_a_min, int x_a);
  void ag_psp_RA_group_ai_k(double & sinx_ik_a_min, int x_a);
  void ac_psp_RA_group_ai_b(double & sinx_i_ab_min, int x_a);
  void ag_psp_RA_group_ai_b(double & sinx_i_ab_min, int x_a);

  //  dipolesubtraction/phasespace.generation.dipoles.CDST.cpp
  void ac_psp_RA_group_ij_k_massive(int x_a);
  void ag_psp_RA_group_ij_k_massive(int x_a);
  void ac_psp_RA_group_ij_a_massive(double & sinx_ij_a_min, int x_a);
  void ag_psp_RA_group_ij_a_massive(double & sinx_ij_a_min, int x_a);
  void ac_psp_RA_group_ai_k_massive(double & sinx_ik_a_min, int x_a);
  void ag_psp_RA_group_ai_k_massive(double & sinx_ik_a_min, int x_a);

  

  



  // To create pseudo-dipoles if needed:
  phasespace_set * fake_psi;


  bool error_readin;
  string dir_MCweights;

  randommanager random_manager;
  vector<randomvariable*> random_psp;

  vector<randomvariable*> QT_random_z;
  randomvariable* QT_random_z1;
  randomvariable* QT_random_z2;
  randomvariable* QT_random_qt;

  vector<double> sran;

  // random numbers could become part of multichannel/importancesampling sets !!!
  vector<double> r;

  vector<double> random_MC;
  double n_random_MC;
  vector<double> random_MC_tau;
  double n_random_MC_tau;
  vector<double> random_tau;
  double n_random_tau;
  vector<double> random_x12;
  double n_random_x12;
  // Later identify with  random_qT  and  n_random_qT  from power correction contribution !!!
  vector<double> random_qTres;
  double n_random_qTres;
  vector<vector<double> > random_z;
  double n_random_z;
  vector<double> random_qT;
  double n_random_qT;

  Logger * logger; // ???

  int switch_n_events_opt;

  int switch_qTcut;


  // MC parameters fixed for the whole run, maybe up to modifications depending on optimization procedure

  vector<vector<vector<string> > > MC_IS_name_type;
  vector<string> MC_IS_name_list;
  map <int, vector<int> > map_list_to_type_MC_IS;
  map <vector<int>, int > map_type_to_list_MC_IS;

  vector<vector<vector<int> > > c_p;
  vector<vector<vector<int> > > c_f;
  vector<vector<vector<int> > > c_t;
  vector<vector<vector<int> > > c_d;

  vector<vector<double> > v_smin;
  vector<vector<double> > v_smax;

  vector<vector<double> > g_p;
  vector<vector<double> > g_f;
  vector<vector<double> > g_t;
  vector<vector<double> > g_d;

  vector<vector<vector<int> > > needed_v_smin;
  vector<vector<vector<int> > > needed_v_smax;

  vector<vector<vector<int> > > needed_g_p;
  vector<vector<vector<int> > > needed_g_f;
  vector<vector<vector<int> > > needed_g_t;
  vector<vector<vector<int> > > needed_g_d;


  // variables changing at each phase-space point
  vector<vector<fourvector> > xbp_all;
  vector<vector<double> > xbs_all;
  vector<vector<double> > xbsqrts_all;


  // MC parameters fixed for the whole run
  int n_events_max;
  int n_events_min;
  int n_step;
  int n_repetition;
  
  int n_events_MC_opt;
  int opt_n_events_min;

  int xb_max;
  int xb_max_dipoles;
  int xb_out_dipoles;
  int no_random;

  vector<int> no_random_dipole;

  vector<vector<fourvector> > start_xbp_all;
  vector<vector<double> > start_xbs_all;
  vector<vector<double> > start_xbsqrts_all;

  vector<vector<fourvector> > corrected_xbp_all;
  vector<vector<double> > corrected_xbs_all;
  vector<vector<double> > corrected_xbsqrts_all;


  int n_dipoles;
  vector<double> dipole_sinx_min;
  vector<double> dipole_x;

  double s_had;
  double tau_0_s_had;

  double nuxs;
  double nuxt;
  double exp_pdf;
  
  double exp_ij_k_y;
  double exp_ij_k_z;
  double exp_ij_a_x;
  double exp_ij_a_z;
  double exp_ai_k_x;
  double exp_ai_k_u;
  double exp_ai_b_x;
  double exp_ai_b_v;
  double map_technical_r;
  double map_technical_s;
  double map_technical_t;
  double map_technical_x;
  double mass0;

  vector<vector<double> > smin_opt;
  vector<vector<double> > sqrtsmin_opt;

  double E;
  double tau_0;
  int coll_choice;
  double min_qTcut;

  vector<double> mapping_cut_pT;

  double g_onshell_decay;

  vector<double> tau_0_num;

  // MC parameters modified at each optimization step
  int MCweight_lc_min;
  int switch_use_alpha_after_IS;
  int switch_step_mode_grid;
  int x_alpha_it_min;


  // model parameters, fixed for the whole run
  // could always be taken from model_set msi, but one might want to use modified values for mappings ???
  vector<double> M;
  vector<double> M2;
  vector<double> Gamma;
  vector<double_complex> cM2;
  vector<double> map_Gamma;
  vector<double> reg_Gamma;


  // input/output filenames, fixed for the whole run
  string filename_weight_readin;
  string filename_weight_writeout;

  
  // MC optimization parameters
  int switch_IS_mode_phasespace;
  string weight_in_directory;


  // MC parameters changing on runtime (not used so far !!!)
  int MC_optswitch;

  // number of channels within respective dipole
  vector<int> MC_n_channel_phasespace;
  // number of channels summed up to respective dipole
  vector<int> MC_sum_channel_phasespace;
  // number of channels in total
  int MC_n_channel;
  // selected channel within respective dipole (referring to MC_n_channel_phasespace[x_a])
  int MC_channel_phasespace;
  // selected channel in total (referring to MC_n_channel)
  int MC_channel;


  double hcf;
  double ps_factor;

  double g_tot;
  double g_global_NWA;

  double g_MC;
  double g_pdf;
  double g_tau;
  double g_x1x2;

  double g_qTres;
  double g_qT;

  double MC_g_IS_global;

  int active_optimization;
  int end_optimization;

  int MC_opt_end;
  int IS_opt_end;
  int n_tau_MC_channel;
  vector<int> tau_MC_map;
  vector<vector<double> > tau_MC_tau_gamma;

  
  vector<multichannel_input * > input_MC;
  
  multichannel_input input_MC_phasespace;
  multichannel_set MC_phasespace;

  // IS-multichannelling with optimization
  multichannel_input input_MC_tau;
  multichannel_set MC_tau;

  // IS-multichannelling inside dipole mappings with optimization
  multichannel_input input_MC_x_dipole;
  
  vector<vector<int> > MC_x_dipole_mapping;
  vector<multichannel_set> MC_x_dipole;

  vector<importancesampling_input * > input_IS;

  importancesampling_input input_IS_tau;
  importancesampling_set IS_tau;

  importancesampling_input input_IS_x1x2;
  importancesampling_set IS_x1x2;

  importancesampling_input input_IS_z1z2;
  vector<importancesampling_set> IS_z1z2;

  // Use input_IS_qTres and IS_qTres also for recoil in power correction contribution !!!
  importancesampling_input input_IS_qTres;
  importancesampling_set IS_qTres;

  importancesampling_input input_IS_PS_p;
  importancesampling_input input_IS_PS_f;
  importancesampling_input input_IS_PS_d_phi;
  importancesampling_input input_IS_PS_d_costheta;
  importancesampling_input input_IS_PS_t_t;
  importancesampling_input input_IS_PS_t_phi;
  importancesampling_input input_IS_PS_dipole_xy;
  importancesampling_input input_IS_PS_dipole_uvz;
  importancesampling_input input_IS_PS_dipole_phi;

  int z1z2_opt_end;


  // Should maybe be copied/shifted to esi (as possibly also z_coll, etc.):
  double boost;
  vector<double> x_pdf;

  // SK: Is that one used ??? clarify !!! Only this one should be used in the future !!!
  vector<vector<double> > xz_pdf;

  vector<double> z_coll;
  // Remove !!!
  vector<vector<double> > all_xz_coll_pdf;
  vector<double> g_z_coll;

  fourvector recoil_system;
  double recoil_q;
  double recoil_q2;
  double recoil_qT;
  double recoil_qT2;
  double recoil_r;
  double recoil_phi;
  int recoil_rcut;
  
  // QT definitions - should later be removed and unified with the ones used in CA !!!
  double QT_qt2;
  double QT_jacqt2;

  
  int switch_resummation;
  int dynamical_Qres;
  double Qres;
  double Qres_prefactor;
  // Check how the resummation-related variables are connected !!!
  int switch_dynamic_Qres;


  double lambda_qt2; // for generation of qT in a resummed computation

  // SK: QT_random_qt2 needed any longer ???
  double QT_random_qt2;
  // careful: contains z1,z2, NOT x1/z1,x2/z2
  vector<double> QT_zx_pdf;
  vector<double> QT_xz_pdf;

  vector<double> zz_pdf;
  vector<double> z_pdf;
  vector<vector<int> > zall_pdf;
  double QT_g_z1;
  double QT_g_z2;
  double QT_eps;
  double QT_random_IS;
  double QT_g_IS_;
  //



  long long i_gen;
  long long i_rej;
  long long i_acc;
  long long i_nan;
  long long i_tec;

  long long last_step_mode;
  long long * i_step_mode;

  bool enforce_step;
  bool enforce_int_end;

  long long i_gen_step;
  long long i_acc_step;
  int i_step;
  int n_step_max;
  
  int random_seed;

  int RA_techcut;
  int RA_x_a;

  double cut_technical;

  // needed ? Maybe only once when setting parameters ???
  contribution_set *csi;
  model_set *msi;
  event_set *esi;
  user_defined *user;

  string run_mode;

  //  int switch_console_output_tau_0;

  int switch_console_output_phasespace_issue;

  int switch_off_RS_mapping;
  int switch_off_RS_mapping_ij_k;
  int switch_off_RS_mapping_ij_a;
  int switch_off_RS_mapping_ai_k;
  int switch_off_RS_mapping_ai_b;

  int n_random_per_psp;
  int switch_off_random_generator;
  int switch_IO_generator;

  int switch_output_weights;
  
  random_number_generator rng;

  int n_shift_run;
  int add_random_per_psp;

  bool identical_psp;
};

#endif
