#include "header.hpp"

#include "ppttx20.phasespace.set.hpp"

ppttx20_phasespace_set::~ppttx20_phasespace_set(){
  static Logger logger("ppttx20_phasespace_set::~ppttx20_phasespace_set");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;


  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::optimize_minv_born(){
  static Logger logger("ppttx20_phasespace_set::optimize_minv_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

#include "specify.optimize.phasespace.minv.born.cxx"

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

int ppttx20_phasespace_set::determination_MCchannels_born(int x_a){
  static Logger logger("ppttx20_phasespace_set::determination_MCchannels_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  int n_channel = 0;
  if (csi->phasespace_order_alpha_s[x_a] == 2 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if      (csi->no_process_parton[x_a] == 0){n_channel = 3;}
    else if (csi->no_process_parton[x_a] == 1){n_channel = 3;}
    else if (csi->no_process_parton[x_a] == 2){n_channel = 1;}
    else if (csi->no_process_parton[x_a] == 3){n_channel = 1;}
    else if (csi->no_process_parton[x_a] == 4){n_channel = 1;}
  }
  return n_channel;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ac_tau_psp_born(int x_a, vector<int> & tau_MC_map){
  static Logger logger("ppttx20_phasespace_set::ac_tau_psp_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 2 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if      (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] == 1){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 2){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 3){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 4){tau_MC_map = vector<int> {  0};}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ax_psp_born(int x_a){
  static Logger logger("ppttx20_phasespace_set::ax_psp_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 2 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] == 1){ax_psp_200_gg_ttx(x_a);}
    else if (csi->no_process_parton[x_a] == 2){ax_psp_200_ddx_ttx(x_a);}
    else if (csi->no_process_parton[x_a] == 3){ax_psp_200_uux_ttx(x_a);}
    else if (csi->no_process_parton[x_a] == 4){ax_psp_200_bbx_ttx(x_a);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ac_psp_born(int x_a, int channel){
  static Logger logger("ppttx20_phasespace_set::ac_psp_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 2 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] == 1){ac_psp_200_gg_ttx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 2){ac_psp_200_ddx_ttx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 3){ac_psp_200_uux_ttx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 4){ac_psp_200_bbx_ttx(x_a, channel);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ag_psp_born(int x_a, int zero){
  static Logger logger("ppttx20_phasespace_set::ag_psp_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 2 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] == 1){ag_psp_200_gg_ttx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 2){ag_psp_200_ddx_ttx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 3){ag_psp_200_uux_ttx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 4){ag_psp_200_bbx_ttx(x_a, zero);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::optimize_minv_real(){
  static Logger logger("ppttx20_phasespace_set::optimize_minv_real");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

#include "specify.optimize.phasespace.minv.real.cxx"

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

int ppttx20_phasespace_set::determination_MCchannels_real(int x_a){
  static Logger logger("ppttx20_phasespace_set::determination_MCchannels_real");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  int n_channel = 0;
  if (csi->phasespace_order_alpha_s[x_a] == 3 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if      (csi->no_process_parton[x_a] ==  0){n_channel = 18;}
    else if (csi->no_process_parton[x_a] ==  1){n_channel = 15;}
    else if (csi->no_process_parton[x_a] ==  2){n_channel = 5;}
    else if (csi->no_process_parton[x_a] ==  3){n_channel = 5;}
    else if (csi->no_process_parton[x_a] ==  4){n_channel = 5;}
    else if (csi->no_process_parton[x_a] ==  5){n_channel = 5;}
    else if (csi->no_process_parton[x_a] ==  6){n_channel = 5;}
    else if (csi->no_process_parton[x_a] ==  7){n_channel = 5;}
    else if (csi->no_process_parton[x_a] ==  8){n_channel = 5;}
    else if (csi->no_process_parton[x_a] ==  9){n_channel = 5;}
    else if (csi->no_process_parton[x_a] == 10){n_channel = 5;}
  }
  return n_channel;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ac_tau_psp_real(int x_a, vector<int> & tau_MC_map){
  static Logger logger("ppttx20_phasespace_set::ac_tau_psp_real");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 3 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if      (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] ==  1){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  2){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  3){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  4){tau_MC_map = vector<int> {  0,  -5};}
    else if (csi->no_process_parton[x_a] ==  5){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  6){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  7){tau_MC_map = vector<int> {  0,  -5};}
    else if (csi->no_process_parton[x_a] ==  8){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  9){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 10){tau_MC_map = vector<int> {  0};}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ax_psp_real(int x_a){
  static Logger logger("ppttx20_phasespace_set::ax_psp_real");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 3 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] ==  1){ax_psp_300_gg_ttxg(x_a);}
    else if (csi->no_process_parton[x_a] ==  2){ax_psp_300_gd_ttxd(x_a);}
    else if (csi->no_process_parton[x_a] ==  3){ax_psp_300_gu_ttxu(x_a);}
    else if (csi->no_process_parton[x_a] ==  4){ax_psp_300_gb_ttxb(x_a);}
    else if (csi->no_process_parton[x_a] ==  5){ax_psp_300_gdx_ttxdx(x_a);}
    else if (csi->no_process_parton[x_a] ==  6){ax_psp_300_gux_ttxux(x_a);}
    else if (csi->no_process_parton[x_a] ==  7){ax_psp_300_gbx_ttxbx(x_a);}
    else if (csi->no_process_parton[x_a] ==  8){ax_psp_300_ddx_ttxg(x_a);}
    else if (csi->no_process_parton[x_a] ==  9){ax_psp_300_uux_ttxg(x_a);}
    else if (csi->no_process_parton[x_a] == 10){ax_psp_300_bbx_ttxg(x_a);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ac_psp_real(int x_a, int channel){
  static Logger logger("ppttx20_phasespace_set::ac_psp_real");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 3 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] ==  1){ac_psp_300_gg_ttxg(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  2){ac_psp_300_gd_ttxd(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  3){ac_psp_300_gu_ttxu(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  4){ac_psp_300_gb_ttxb(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  5){ac_psp_300_gdx_ttxdx(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  6){ac_psp_300_gux_ttxux(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  7){ac_psp_300_gbx_ttxbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  8){ac_psp_300_ddx_ttxg(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  9){ac_psp_300_uux_ttxg(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 10){ac_psp_300_bbx_ttxg(x_a, channel);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ag_psp_real(int x_a, int zero){
  static Logger logger("ppttx20_phasespace_set::ag_psp_real");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 3 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] ==  1){ag_psp_300_gg_ttxg(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  2){ag_psp_300_gd_ttxd(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  3){ag_psp_300_gu_ttxu(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  4){ag_psp_300_gb_ttxb(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  5){ag_psp_300_gdx_ttxdx(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  6){ag_psp_300_gux_ttxux(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  7){ag_psp_300_gbx_ttxbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  8){ag_psp_300_ddx_ttxg(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  9){ag_psp_300_uux_ttxg(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 10){ag_psp_300_bbx_ttxg(x_a, zero);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::optimize_minv_doublereal(){
  static Logger logger("ppttx20_phasespace_set::optimize_minv_doublereal");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

#include "specify.optimize.phasespace.minv.doublereal.cxx"

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

int ppttx20_phasespace_set::determination_MCchannels_doublereal(int x_a){
  static Logger logger("ppttx20_phasespace_set::determination_MCchannels_doublereal");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  int n_channel = 0;
  if (csi->phasespace_order_alpha_s[x_a] == 4 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if      (csi->no_process_parton[x_a] ==  0){n_channel = 160;}
    else if (csi->no_process_parton[x_a] ==  1){n_channel = 105;}
    else if (csi->no_process_parton[x_a] ==  2){n_channel = 35;}
    else if (csi->no_process_parton[x_a] ==  3){n_channel = 35;}
    else if (csi->no_process_parton[x_a] ==  4){n_channel = 35;}
    else if (csi->no_process_parton[x_a] ==  5){n_channel = 35;}
    else if (csi->no_process_parton[x_a] ==  6){n_channel = 35;}
    else if (csi->no_process_parton[x_a] ==  7){n_channel = 35;}
    else if (csi->no_process_parton[x_a] ==  8){n_channel = 35;}
    else if (csi->no_process_parton[x_a] ==  9){n_channel = 35;}
    else if (csi->no_process_parton[x_a] == 10){n_channel = 35;}
    else if (csi->no_process_parton[x_a] == 11){n_channel = 14;}
    else if (csi->no_process_parton[x_a] == 12){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 13){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 14){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 16){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 17){n_channel = 35;}
    else if (csi->no_process_parton[x_a] == 18){n_channel = 14;}
    else if (csi->no_process_parton[x_a] == 19){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 20){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 21){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 22){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 23){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 25){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 27){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 28){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 29){n_channel = 14;}
    else if (csi->no_process_parton[x_a] == 30){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 31){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 32){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 34){n_channel = 35;}
    else if (csi->no_process_parton[x_a] == 35){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 36){n_channel = 14;}
    else if (csi->no_process_parton[x_a] == 37){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 38){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 39){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 40){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 42){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 43){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 44){n_channel = 14;}
    else if (csi->no_process_parton[x_a] == 45){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 46){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 47){n_channel = 35;}
    else if (csi->no_process_parton[x_a] == 48){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 49){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 50){n_channel = 14;}
    else if (csi->no_process_parton[x_a] == 51){n_channel = 14;}
    else if (csi->no_process_parton[x_a] == 52){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 53){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 54){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 56){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 57){n_channel = 14;}
    else if (csi->no_process_parton[x_a] == 58){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 59){n_channel = 7;}
    else if (csi->no_process_parton[x_a] == 60){n_channel = 14;}
  }
  return n_channel;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ac_tau_psp_doublereal(int x_a, vector<int> & tau_MC_map){
  static Logger logger("ppttx20_phasespace_set::ac_tau_psp_doublereal");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 4 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if      (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] ==  1){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  2){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  3){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  4){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  5){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  6){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  7){tau_MC_map = vector<int> {  0,  -5};}
    else if (csi->no_process_parton[x_a] ==  8){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] ==  9){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 10){tau_MC_map = vector<int> {  0,  -5};}
    else if (csi->no_process_parton[x_a] == 11){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 12){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 13){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 14){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 16){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 17){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 18){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 19){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 20){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 21){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 22){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 23){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 25){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 27){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 28){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 29){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 30){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 31){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 32){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 34){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 35){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 36){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 37){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 38){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 39){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 40){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 42){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 43){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 44){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 45){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 46){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 47){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 48){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 49){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 50){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 51){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 52){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 53){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 54){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 56){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 57){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 58){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 59){tau_MC_map = vector<int> {  0};}
    else if (csi->no_process_parton[x_a] == 60){tau_MC_map = vector<int> {  0};}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ax_psp_doublereal(int x_a){
  static Logger logger("ppttx20_phasespace_set::ax_psp_doublereal");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 4 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] ==  1){ax_psp_400_gg_ttxgg(x_a);}
    else if (csi->no_process_parton[x_a] ==  2){ax_psp_400_gg_ttxddx(x_a);}
    else if (csi->no_process_parton[x_a] ==  3){ax_psp_400_gg_ttxuux(x_a);}
    else if (csi->no_process_parton[x_a] ==  4){ax_psp_400_gg_ttxbbx(x_a);}
    else if (csi->no_process_parton[x_a] ==  5){ax_psp_400_gd_ttxgd(x_a);}
    else if (csi->no_process_parton[x_a] ==  6){ax_psp_400_gu_ttxgu(x_a);}
    else if (csi->no_process_parton[x_a] ==  7){ax_psp_400_gb_ttxgb(x_a);}
    else if (csi->no_process_parton[x_a] ==  8){ax_psp_400_gdx_ttxgdx(x_a);}
    else if (csi->no_process_parton[x_a] ==  9){ax_psp_400_gux_ttxgux(x_a);}
    else if (csi->no_process_parton[x_a] == 10){ax_psp_400_gbx_ttxgbx(x_a);}
    else if (csi->no_process_parton[x_a] == 11){ax_psp_400_dd_ttxdd(x_a);}
    else if (csi->no_process_parton[x_a] == 12){ax_psp_400_du_ttxdu(x_a);}
    else if (csi->no_process_parton[x_a] == 13){ax_psp_400_ds_ttxds(x_a);}
    else if (csi->no_process_parton[x_a] == 14){ax_psp_400_dc_ttxdc(x_a);}
    else if (csi->no_process_parton[x_a] == 16){ax_psp_400_db_ttxdb(x_a);}
    else if (csi->no_process_parton[x_a] == 17){ax_psp_400_ddx_ttxgg(x_a);}
    else if (csi->no_process_parton[x_a] == 18){ax_psp_400_ddx_ttxddx(x_a);}
    else if (csi->no_process_parton[x_a] == 19){ax_psp_400_ddx_ttxuux(x_a);}
    else if (csi->no_process_parton[x_a] == 20){ax_psp_400_ddx_ttxssx(x_a);}
    else if (csi->no_process_parton[x_a] == 21){ax_psp_400_ddx_ttxccx(x_a);}
    else if (csi->no_process_parton[x_a] == 22){ax_psp_400_ddx_ttxbbx(x_a);}
    else if (csi->no_process_parton[x_a] == 23){ax_psp_400_dux_ttxdux(x_a);}
    else if (csi->no_process_parton[x_a] == 25){ax_psp_400_dsx_ttxdsx(x_a);}
    else if (csi->no_process_parton[x_a] == 27){ax_psp_400_dcx_ttxdcx(x_a);}
    else if (csi->no_process_parton[x_a] == 28){ax_psp_400_dbx_ttxdbx(x_a);}
    else if (csi->no_process_parton[x_a] == 29){ax_psp_400_uu_ttxuu(x_a);}
    else if (csi->no_process_parton[x_a] == 30){ax_psp_400_uc_ttxuc(x_a);}
    else if (csi->no_process_parton[x_a] == 31){ax_psp_400_ub_ttxub(x_a);}
    else if (csi->no_process_parton[x_a] == 32){ax_psp_400_udx_ttxudx(x_a);}
    else if (csi->no_process_parton[x_a] == 34){ax_psp_400_uux_ttxgg(x_a);}
    else if (csi->no_process_parton[x_a] == 35){ax_psp_400_uux_ttxddx(x_a);}
    else if (csi->no_process_parton[x_a] == 36){ax_psp_400_uux_ttxuux(x_a);}
    else if (csi->no_process_parton[x_a] == 37){ax_psp_400_uux_ttxssx(x_a);}
    else if (csi->no_process_parton[x_a] == 38){ax_psp_400_uux_ttxccx(x_a);}
    else if (csi->no_process_parton[x_a] == 39){ax_psp_400_uux_ttxbbx(x_a);}
    else if (csi->no_process_parton[x_a] == 40){ax_psp_400_usx_ttxusx(x_a);}
    else if (csi->no_process_parton[x_a] == 42){ax_psp_400_ucx_ttxucx(x_a);}
    else if (csi->no_process_parton[x_a] == 43){ax_psp_400_ubx_ttxubx(x_a);}
    else if (csi->no_process_parton[x_a] == 44){ax_psp_400_bb_ttxbb(x_a);}
    else if (csi->no_process_parton[x_a] == 45){ax_psp_400_bdx_ttxbdx(x_a);}
    else if (csi->no_process_parton[x_a] == 46){ax_psp_400_bux_ttxbux(x_a);}
    else if (csi->no_process_parton[x_a] == 47){ax_psp_400_bbx_ttxgg(x_a);}
    else if (csi->no_process_parton[x_a] == 48){ax_psp_400_bbx_ttxddx(x_a);}
    else if (csi->no_process_parton[x_a] == 49){ax_psp_400_bbx_ttxuux(x_a);}
    else if (csi->no_process_parton[x_a] == 50){ax_psp_400_bbx_ttxbbx(x_a);}
    else if (csi->no_process_parton[x_a] == 51){ax_psp_400_dxdx_ttxdxdx(x_a);}
    else if (csi->no_process_parton[x_a] == 52){ax_psp_400_dxux_ttxdxux(x_a);}
    else if (csi->no_process_parton[x_a] == 53){ax_psp_400_dxsx_ttxdxsx(x_a);}
    else if (csi->no_process_parton[x_a] == 54){ax_psp_400_dxcx_ttxdxcx(x_a);}
    else if (csi->no_process_parton[x_a] == 56){ax_psp_400_dxbx_ttxdxbx(x_a);}
    else if (csi->no_process_parton[x_a] == 57){ax_psp_400_uxux_ttxuxux(x_a);}
    else if (csi->no_process_parton[x_a] == 58){ax_psp_400_uxcx_ttxuxcx(x_a);}
    else if (csi->no_process_parton[x_a] == 59){ax_psp_400_uxbx_ttxuxbx(x_a);}
    else if (csi->no_process_parton[x_a] == 60){ax_psp_400_bxbx_ttxbxbx(x_a);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ac_psp_doublereal(int x_a, int channel){
  static Logger logger("ppttx20_phasespace_set::ac_psp_doublereal");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 4 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] ==  1){ac_psp_400_gg_ttxgg(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  2){ac_psp_400_gg_ttxddx(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  3){ac_psp_400_gg_ttxuux(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  4){ac_psp_400_gg_ttxbbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  5){ac_psp_400_gd_ttxgd(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  6){ac_psp_400_gu_ttxgu(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  7){ac_psp_400_gb_ttxgb(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  8){ac_psp_400_gdx_ttxgdx(x_a, channel);}
    else if (csi->no_process_parton[x_a] ==  9){ac_psp_400_gux_ttxgux(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 10){ac_psp_400_gbx_ttxgbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 11){ac_psp_400_dd_ttxdd(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 12){ac_psp_400_du_ttxdu(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 13){ac_psp_400_ds_ttxds(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 14){ac_psp_400_dc_ttxdc(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 16){ac_psp_400_db_ttxdb(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 17){ac_psp_400_ddx_ttxgg(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 18){ac_psp_400_ddx_ttxddx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 19){ac_psp_400_ddx_ttxuux(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 20){ac_psp_400_ddx_ttxssx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 21){ac_psp_400_ddx_ttxccx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 22){ac_psp_400_ddx_ttxbbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 23){ac_psp_400_dux_ttxdux(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 25){ac_psp_400_dsx_ttxdsx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 27){ac_psp_400_dcx_ttxdcx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 28){ac_psp_400_dbx_ttxdbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 29){ac_psp_400_uu_ttxuu(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 30){ac_psp_400_uc_ttxuc(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 31){ac_psp_400_ub_ttxub(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 32){ac_psp_400_udx_ttxudx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 34){ac_psp_400_uux_ttxgg(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 35){ac_psp_400_uux_ttxddx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 36){ac_psp_400_uux_ttxuux(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 37){ac_psp_400_uux_ttxssx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 38){ac_psp_400_uux_ttxccx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 39){ac_psp_400_uux_ttxbbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 40){ac_psp_400_usx_ttxusx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 42){ac_psp_400_ucx_ttxucx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 43){ac_psp_400_ubx_ttxubx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 44){ac_psp_400_bb_ttxbb(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 45){ac_psp_400_bdx_ttxbdx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 46){ac_psp_400_bux_ttxbux(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 47){ac_psp_400_bbx_ttxgg(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 48){ac_psp_400_bbx_ttxddx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 49){ac_psp_400_bbx_ttxuux(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 50){ac_psp_400_bbx_ttxbbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 51){ac_psp_400_dxdx_ttxdxdx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 52){ac_psp_400_dxux_ttxdxux(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 53){ac_psp_400_dxsx_ttxdxsx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 54){ac_psp_400_dxcx_ttxdxcx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 56){ac_psp_400_dxbx_ttxdxbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 57){ac_psp_400_uxux_ttxuxux(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 58){ac_psp_400_uxcx_ttxuxcx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 59){ac_psp_400_uxbx_ttxuxbx(x_a, channel);}
    else if (csi->no_process_parton[x_a] == 60){ac_psp_400_bxbx_ttxbxbx(x_a, channel);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void ppttx20_phasespace_set::ag_psp_doublereal(int x_a, int zero){
  static Logger logger("ppttx20_phasespace_set::ag_psp_doublereal");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (csi->phasespace_order_alpha_s[x_a] == 4 && csi->phasespace_order_alpha_e[x_a] == 0 && csi->phasespace_order_interference[x_a] == 0){
    if (csi->no_process_parton[x_a] == 0){}
    else if (csi->no_process_parton[x_a] ==  1){ag_psp_400_gg_ttxgg(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  2){ag_psp_400_gg_ttxddx(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  3){ag_psp_400_gg_ttxuux(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  4){ag_psp_400_gg_ttxbbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  5){ag_psp_400_gd_ttxgd(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  6){ag_psp_400_gu_ttxgu(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  7){ag_psp_400_gb_ttxgb(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  8){ag_psp_400_gdx_ttxgdx(x_a, zero);}
    else if (csi->no_process_parton[x_a] ==  9){ag_psp_400_gux_ttxgux(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 10){ag_psp_400_gbx_ttxgbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 11){ag_psp_400_dd_ttxdd(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 12){ag_psp_400_du_ttxdu(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 13){ag_psp_400_ds_ttxds(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 14){ag_psp_400_dc_ttxdc(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 16){ag_psp_400_db_ttxdb(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 17){ag_psp_400_ddx_ttxgg(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 18){ag_psp_400_ddx_ttxddx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 19){ag_psp_400_ddx_ttxuux(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 20){ag_psp_400_ddx_ttxssx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 21){ag_psp_400_ddx_ttxccx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 22){ag_psp_400_ddx_ttxbbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 23){ag_psp_400_dux_ttxdux(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 25){ag_psp_400_dsx_ttxdsx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 27){ag_psp_400_dcx_ttxdcx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 28){ag_psp_400_dbx_ttxdbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 29){ag_psp_400_uu_ttxuu(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 30){ag_psp_400_uc_ttxuc(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 31){ag_psp_400_ub_ttxub(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 32){ag_psp_400_udx_ttxudx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 34){ag_psp_400_uux_ttxgg(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 35){ag_psp_400_uux_ttxddx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 36){ag_psp_400_uux_ttxuux(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 37){ag_psp_400_uux_ttxssx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 38){ag_psp_400_uux_ttxccx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 39){ag_psp_400_uux_ttxbbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 40){ag_psp_400_usx_ttxusx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 42){ag_psp_400_ucx_ttxucx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 43){ag_psp_400_ubx_ttxubx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 44){ag_psp_400_bb_ttxbb(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 45){ag_psp_400_bdx_ttxbdx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 46){ag_psp_400_bux_ttxbux(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 47){ag_psp_400_bbx_ttxgg(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 48){ag_psp_400_bbx_ttxddx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 49){ag_psp_400_bbx_ttxuux(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 50){ag_psp_400_bbx_ttxbbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 51){ag_psp_400_dxdx_ttxdxdx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 52){ag_psp_400_dxux_ttxdxux(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 53){ag_psp_400_dxsx_ttxdxsx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 54){ag_psp_400_dxcx_ttxdxcx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 56){ag_psp_400_dxbx_ttxdxbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 57){ag_psp_400_uxux_ttxuxux(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 58){ag_psp_400_uxcx_ttxuxcx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 59){ag_psp_400_uxbx_ttxuxbx(x_a, zero);}
    else if (csi->no_process_parton[x_a] == 60){ag_psp_400_bxbx_ttxbxbx(x_a, zero);}
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}
