#include "header.hpp"
#ifdef OPENLOOPS

void amplitude_OpenLoops_set::calculate_ME2_ioperator_VA_QEW(){
  Logger logger("amplitude_OpenLoops_set::calculate_ME2_ioperator_VA_QEW");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  static int initialization = 1;
  static vector<double> I_ME2_emitter(osi->VA_ioperator->size());
  if (initialization == 1){
    for (int i_a = 0; i_a < osi->VA_ioperator->size(); i_a++){
      osi->VA_ME2_cf[i_a].resize((*(osi->VA_ioperator))[i_a].size());
      osi->VA_I_ME2_cf[i_a].resize((*(osi->VA_ioperator))[i_a].size());
    }
    initialization = 0;
  }

  set_phasespacepoint();
  ol_evaluate_tree(1, P_OpenLoops, &osi->VA_b_ME2);

  for (int i_a = 0; i_a < osi->VA_ioperator->size(); i_a++){
    for (int j_a = 0; j_a < (*(osi->VA_ioperator))[i_a].size(); j_a++){
      osi->VA_ME2_cf[i_a][j_a] = (*(osi->VA_ioperator))[i_a][j_a].charge_factor() * osi->VA_b_ME2;
      ///      logger << LOG_INFO << "(*(osi->VA_ioperator))[" << i_a << "][" << j_a << "].charge_factor() = " << (*(osi->VA_ioperator))[i_a][j_a].charge_factor() << endl;
    }
  }

  osi->calculate_ioperator_QEW();

  for (int i_a = 0; i_a < osi->VA_ioperator->size(); i_a++){
    I_ME2_emitter[i_a] = accumulate(osi->VA_I_ME2_cf[i_a].begin(), osi->VA_I_ME2_cf[i_a].end(), 0.);
    //    logger << LOG_DEBUG_VERBOSE << "I_ME2_emitter[" << i_a << "] = " << I_ME2_emitter[i_a] << endline_flush;

  }
  osi->VA_I_ME2 = accumulate(I_ME2_emitter.begin(), I_ME2_emitter.end(), 0.);
  //  logger << LOG_DEBUG_VERBOSE << "osi->VA_I_ME2 = " << osi->VA_I_ME2 << endline_flush;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void amplitude_OpenLoops_set::calculate_ME2_VA_QEW(){
  Logger logger("amplitude_OpenLoops_set::calculate_ME2_VA_QEW");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (osi->switch_VI == 0 || osi->switch_VI == 1){

    set_phasespacepoint();
    set_mu_reg();
    set_mu_and_delta_VA();
    /*
    ol_setparameter_double(OL_mu_ren, osi->var_mu_ren);
    //  ol_setparameter_double(OL_mu_reg, osi->var_mu_ren);
    logger << LOG_DEBUG_VERBOSE << "osi->user->double_value[osi->user->double_map[mureg]] = " << osi->user->double_value[osi->user->double_map["mu_reg"]] << endline_flush;
    if (osi->user->double_value[osi->user->double_map["mu_reg"]] == 0.){ol_setparameter_double(OL_mu_reg, osi->var_mu_ren);}
    else if (osi->user->double_value[osi->user->double_map["mu_reg"]] == -1.){ol_setparameter_double(OL_mu_reg, osi->var_mu_ren);}
    else {ol_setparameter_double(OL_mu_reg, osi->user->double_value[osi->user->double_map["mu_reg"]]);}
    logger << LOG_DEBUG_VERBOSE << "osi->user->double_value[osi->user->double_map[mureg]] = " << osi->user->double_value[osi->user->double_map["mu_reg"]] << endline_flush;
    */
    //    ol_setparameter_double(fact_uv, one);
    //    ol_setparameter_double(fact_ir, one);

    ol_evaluate_full(1, P_OpenLoops, &osi->VA_b_ME2, M2L1, IRL1, M2L2, IRL2, &acc);
    osi->VA_V_ME2 = M2L1[0];
    ol_evaluate_ct(1, P_OpenLoops, &M2L0, &osi->VA_X_ME2);

    logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_B  = " << setw(23) << setprecision(15) << osi->VA_b_ME2 << endline_flush;
    logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_VX = " << setw(23) << setprecision(15) << osi->VA_V_ME2 + osi->VA_X_ME2 << endline_flush;
    logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_V  = " << setw(23) << setprecision(15) << osi->VA_V_ME2 << endline_flush;
    logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_CT = " << setw(23) << setprecision(15) << osi->VA_X_ME2 << endline_flush;
    logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_VX = " << setw(23) << setprecision(15) << osi->VA_V_ME2 + osi->VA_X_ME2 << endline_flush;


    if (osi->switch_VI == 0){
      if (osi->user->switch_map["I_Munich"]){
	calculate_ME2_ioperator_VA_QEW();
	logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_I  = " << setw(23) << setprecision(15) << osi->VA_I_ME2 << "   (from Munich)" << endline_flush;
      }
      else {
	if (osi->VA_DeltaIR1 == 0. && osi->VA_DeltaIR2 == 0.){osi->VA_I_ME2 = IRL1[0];}
	else {osi->VA_I_ME2 = IRL1[0] + osi->VA_DeltaIR1 * IRL1[1] + osi->VA_DeltaIR2 * IRL1[2];}
	logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_I  = " << setw(23) << setprecision(15) << osi->VA_I_ME2 << "   (from OpenLoops)" << endline_flush;
      }

      //  osi->VA_I_ME2 = i_DeltaIR1 * IR1 + osi->VA_DeltaIR2 * IR2; // !!! I-operator switched off !!!
    }
    else if (osi->switch_VI == 1){
      osi->VA_I_ME2 = 0.;
    }
    logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_I  = " << setw(23) << setprecision(15) << osi->VA_I_ME2 << endline_flush;

    if (osi->switch_CV){
      for (int s = 0; s < osi->n_scales_CV; s++){
	double inv_factor_CV = osi->var_mu_ren_CV[s] / mu_reg;
	/////        double inv_factor_CV = osi->var_mu_ren_CV[s] / osi->var_mu_ren;
        ol_setparameter_double(OL_mu_ren, osi->var_mu_ren_CV[s]);
	ol_setparameter_double(OL_mu_reg, osi->var_mu_ren_CV[s]);
	/*
	if (osi->user->double_value[osi->user->double_map["mu_reg"]] == 0.){ol_setparameter_double(OL_mu_reg, osi->var_mu_ren_CV[s]);}
	else if (osi->user->double_value[osi->user->double_map["mu_reg"]] == -1.){ol_setparameter_double(OL_mu_reg, osi->var_mu_ren);}
	else {ol_setparameter_double(OL_mu_reg, osi->user->double_value[osi->user->double_map["mu_reg"]]);}
	*/
        ol_setparameter_double(fact_uv, inv_factor_CV);
        ol_setparameter_double(fact_ir, inv_factor_CV);
	ol_evaluate_ct(1, P_OpenLoops, &M2L0, &osi->VA_X_ME2_CV[s]);
	logger << LOG_DEBUG_POINT << "OpenLoops:  VA_X_ME2_CV[" << s << "] = " << setw(23) << setprecision(15) << osi->VA_X_ME2_CV[s] + osi->VA_V_ME2 << endline_flush;
      }
    }
    if (osi->switch_TSV){
      for (int i_v = 0; i_v < osi->max_dyn_ren + 1; i_v++){
	for (int i_r = 0; i_r < osi->n_scale_dyn_ren[i_v]; i_r++){
	  double inv_factor_TSV = osi->value_scale_ren[0][i_v][i_r] / mu_reg;
	  /////	    double inv_factor_TSV = osi->value_scale_ren[0][i_v][i_r] / osi->var_mu_ren;
	  ol_setparameter_double(OL_mu_ren, osi->value_scale_ren[0][i_v][i_r]);
	  ol_setparameter_double(OL_mu_reg, osi->value_scale_ren[0][i_v][i_r]);
	  /*
	  if (osi->user->double_value[osi->user->double_map["mu_reg"]] == 0.){ol_setparameter_double(OL_mu_reg, osi->value_scale_ren[0][i_v][i_r]);}
	  else if (osi->user->double_value[osi->user->double_map["mu_reg"]] == -1.){ol_setparameter_double(OL_mu_reg, osi->var_mu_ren);}
	  else {ol_setparameter_double(OL_mu_reg, osi->user->double_value[osi->user->double_map["mu_reg"]]);}
	  */
	  ol_setparameter_double(fact_uv, inv_factor_TSV);
	  ol_setparameter_double(fact_ir, inv_factor_TSV);
	  ol_evaluate_ct(1, P_OpenLoops, &M2L0, &osi->value_ME2term_ren[0][i_v][i_r]);
	  logger << LOG_DEBUG_POINT << "OpenLoops:  VA_VX_ME2_TSV[" << i_v << "][" << i_r << "] = " << osi->value_ME2term_ren[0][i_v][i_r] + osi->VA_V_ME2 << endline_flush;
	  osi->value_ME2term_ren[0][i_v][i_r] += osi->VA_V_ME2 + osi->VA_I_ME2;
	}
      }
    }
  }
  else if (osi->switch_VI == 2){
    logger << LOG_DEBUG_VERBOSE << "OL: osi->VA_I_ME2 = " << osi->VA_I_ME2 << endline_flush;
    //  osi->VA_I_ME2 = 0.;
    //  osi->VA_V_ME2 = 0.;
    //  osi->VA_X_ME2 = 0.;
    //  for (int s = 0; s < osi->n_scales_CV; s++){osi->VA_X_ME2_CV[s] = 0.;}
    //  I-operator evaluation -> osi->VA_I_ME2
    calculate_ME2_ioperator_VA_QEW();
    for (int i_v = 0; i_v < osi->max_dyn_ren + 1; i_v++){
      for (int i_r = 0; i_r < osi->n_scale_dyn_ren[i_v]; i_r++){
        osi->value_ME2term_ren[0][i_v][i_r] = osi->VA_I_ME2;
      }
    }
    logger << LOG_DEBUG << "SK: osi->VA_I_ME2 = " << osi->VA_I_ME2 << endline_flush;
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

#endif
