#include "header.hpp"
#ifdef OPENLOOPS

void amplitude_OpenLoops_set::set_delta_and_mu_Q(){
  Logger logger("amplitude_OpenLoops_set::set_delta_and_mu_Q");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  ol_setparameter_double(pole_uv, osi->VA_DeltaUV);
  ol_setparameter_double(pole_ir1, osi->VA_DeltaIR1);
  ol_setparameter_double(pole_ir2, osi->VA_DeltaIR2);
  mu_Q = (esi->p_parton[0][1] + esi->p_parton[0][2]).m();
  mu_reg = mu_Q;
  ol_setparameter_double(OL_mu_ren, mu_Q);
  ol_setparameter_double(OL_mu_reg, mu_Q);
  ol_setparameter_double(fact_uv, one);
  ol_setparameter_double(fact_ir, one);

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void amplitude_OpenLoops_set::calculate_ME2_VT_QCD(){
  static Logger logger("amplitude_OpenLoops_set::calculate_ME2_VT_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (osi->QT_finalstate_massive_coloured){logger << LOG_WARN << "Process-specific function needs to be called, but has not been defined!" << flush << endl; exit(1);}

  set_phasespacepoint();
  set_delta_and_mu_Q();
  if (!csi->class_contribution_loopinduced){
    // Make sure that this function is not called for NNLO contributions !!! Otherwise:
    // if (osi->initial_diag || (osi->initial_pdf_gq && csi->class_contribution_order == 2)){
    if (osi->initial_diag && !switch_off_H1){
      ol_evaluate_full(1, P_OpenLoops, &osi->VA_b_ME2, M2L1, IRL1, M2L2, IRL2, &acc);
      osi->VA_V_ME2 = M2L1[0];
      // CT_on = 1 should always be set for VT contribution !!!
      //      ol_evaluate_ct(1, P_OpenLoops, &M2L0, &osi->VA_X_ME2);
      logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_B = " << setw(23) << setprecision(15) << osi->VA_b_ME2 << endline_flush;
      logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_VX = " << setw(23) << setprecision(15) << osi->VA_V_ME2 + osi->VA_X_ME2 << "   " << "mu_Q = " << mu_reg << "   " << " = mu_ren = mu_reg" << endline_flush;
      logger << LOG_DEBUG_POINT << "osi->var_alpha_S_reference = " << osi->var_alpha_S_reference << endline_flush;
      logger << LOG_DEBUG_POINT << "osi->alpha_S = " << osi->alpha_S << endline_flush;
      logger << LOG_DEBUG_VERBOSE << "A0 = " << osi->VA_b_ME2 << endline_flush;
      logger << LOG_DEBUG_VERBOSE << "A1 = " << (osi->VA_V_ME2 + osi->VA_X_ME2) << endline_flush;
      logger << LOG_DEBUG_VERBOSE << "msi->alpha_e  = " << msi->alpha_e << endline_flush;

      //  osi->VA_V_ME2 = 2 Re < M0 | M1 >  etc.
      osi->QT_H1_delta_QCD = (osi->VA_V_ME2 + osi->VA_X_ME2) / osi->VA_b_ME2 / (osi->alpha_S / pi);
      // same translation for osi->QT_H1_delta_QCD as in ppllll24_calculate_H2.
      //  (alpha_S / pi) normalization of H1
      if (osi->switch_polenorm == 1){
	if (osi->initial_gg){osi->QT_H1_delta_QCD -= pi2_6 * C_A;}
	else if (osi->initial_qqx){osi->QT_H1_delta_QCD -= pi2_6 * C_F;}
	else {logger << LOG_FATAL << "Wrong process specified." << endline_flush; exit(1);}
      }
      logger << LOG_DEBUG_POINT << "osi->QT_H1_delta_QCD = " << osi->QT_H1_delta_QCD << endline_flush;
      logger << LOG_DEBUG_POINT << "osi->QT_H1_delta_QCD * osi->VA_b_ME2   = " << osi->QT_H1_delta_QCD * osi->VA_b_ME2 << endline_flush;

    }
    else {
      ol_evaluate_tree(1, P_OpenLoops, &osi->VA_b_ME2);      
      osi->QT_H1_delta_QCD = 0.;
    }
  }
  else {
    ol_evaluate_loop2(1, P_OpenLoops, M2L2, &acc);
    osi->VA_b_ME2 = M2L2[0];
    logger << LOG_DEBUG_POINT << "OpenLoops:  ME2_L2I = " << setw(23) << setprecision(15) << osi->VA_b_ME2 << endline_flush;

    if (osi->initial_diag && !switch_off_H1gg){
      calculate_H1gg_QCD_2loop();
    }
    else {
      osi->QT_A0 = osi->VA_b_ME2;
      osi->QT_A1 = 0.;
      osi->VA_V_ME2 = 0.;
      osi->QT_H1_delta_QCD = 0.;
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

#endif
