#include "header.hpp"
#ifdef RECOLA

void amplitude_Recola_set::calculate_ME2_RA_QEW(){
  Logger logger("amplitude_Recola_set::calculate_ME2_RA_QEW");
  logger << LOG_DEBUG_VERBOSE << "called" << flush << endl;

  if (esi->cut_ps[0] > -1){
    double P_Recola[esi->p_parton[0].size()][4];
    set_phasespacepoint(P_Recola);

    compute_process_rcl(1, P_Recola, "LO");
    get_squared_amplitude_rcl(1, csi->contribution_order_alpha_s, "LO", osi->value_ME2term[0]);
    logger << LOG_DEBUG_POINT << "Recola:        R_ME2 = " << setw(23) << setprecision(15) << osi->value_ME2term[0] << endl;
  }
  else {osi->value_ME2term[0] = 0.;}

  osi->calculate_dipole_QEW();

  logger << LOG_DEBUG_VERBOSE << "finished" << flush << endl;
}


void amplitude_Recola_set::calculate_dipole_Acc_QEW(int x_a){
  static Logger logger("amplitude_Recola_set::calculate_dipole_Acc_QEW");
  logger << LOG_DEBUG_VERBOSE << "called" << flush << endl;

  osi->RA_ME2_cc[x_a] = 0.;

  double P_dipole_Recola[esi->p_parton[0].size()][4];
  set_phasespacepoint_dipole(P_dipole_Recola, x_a);

  compute_process_rcl(x_a + 1, P_dipole_Recola, "LO");
  get_squared_amplitude_rcl(x_a + 1, csi->contribution_order_alpha_s, "LO", osi->RA_ME2_cc[x_a]);
  logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_cc[" << x_a << "] = " << setw(23) << setprecision(15) << osi->RA_ME2_cc[x_a] << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << flush << endl;
}


void amplitude_Recola_set::calculate_dipole_Asc_QEW(int x_a, fourvector & Vtensor){
  static Logger logger("amplitude_Recola_set::calculate_dipole_Asc_QEW");
  logger << LOG_DEBUG_VERBOSE << "called" << flush << endl;

  double P_dipole_Recola[esi->p_parton[0].size()][4];
  set_phasespacepoint_dipole(P_dipole_Recola, x_a);

  double_complex MOM[4];
  MOM[0] = Vtensor.x0();
  MOM[1] = Vtensor.x1();
  MOM[2] = Vtensor.x2();
  MOM[3] = Vtensor.x3();
  stringstream temp_ss;
  for (int j = 0; j < 4; j++){
    temp_ss << setw(23) << setprecision(15) << MOM[j] << "   ";
  }

  compute_process_rcl(x_a + 1, P_dipole_Recola, "LO");
  get_squared_amplitude_rcl(x_a + 1, csi->contribution_order_alpha_s, "LO", osi->RA_ME2_cc[x_a]);
  logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_cc[" << x_a << "] = " << setw(23) << setprecision(15) << osi->RA_ME2_cc[x_a] << endl;

  compute_spin_correlation_rcl(x_a + 1, P_dipole_Recola, csi->dipole[x_a].no_A_emitter(), MOM);
  get_spin_correlation_rcl(x_a + 1, csi->contribution_order_alpha_s, osi->RA_ME2_sc[x_a]);
  osi->RA_ME2_sc[x_a] = osi->RA_ME2_sc[x_a] / (-Vtensor.m2());
  logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_sc[" << x_a << "] = " << setw(23) << setprecision(15) << osi->RA_ME2_sc[x_a] << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << flush << endl;
}

#endif
