#include "header.hpp"

void observable_set::initialization_VA(){
  Logger logger("observable_set::initialization_VA");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  n_pc = 1;
  n_ps = 1;
  n_pz = 1;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void observable_set::initialization_specific_VA(){
  Logger logger("observable_set::initialization_specific_VA");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  VA_ME2_cf.resize((*VA_ioperator).size());
  VA_I_ME2_cf.resize((*VA_ioperator).size());
  for (int i_a = 0; i_a < (*VA_ioperator).size(); i_a++){
    VA_ME2_cf[i_a].resize((*VA_ioperator)[i_a].size(), 0.);
    VA_I_ME2_cf[i_a].resize((*VA_ioperator)[i_a].size(), 0.);
  }
  I_ME2_emitter.resize((*VA_ioperator).size());
  
  VA_b_ME2 = 0.;
  VA_V_ME2 = 0.;
  VA_X_ME2 = 0.;
  VA_I_ME2 = 0.;
  VA_X_ME2_CV.resize(n_scales_CV, 0.);

  VA_DeltaUV = 0.;
  VA_DeltaIR1 = 0.;
  VA_DeltaIR2 = 0.;
  VA_delta_flag = 0;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

void observable_set::initialization_CA(){
  Logger logger("observable_set::initialization_CA");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  n_pc = (*CA_collinear).size(); // might be changed later if pdf_selection or pdf_disable is used !
  n_ps = 1;
  n_pz = 3;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void observable_set::initialization_specific_CA(){
  Logger logger("observable_set::initialization_specific_CA");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  CA_ME2_cf.resize((*CA_collinear).size());
  CA_ME2_cf_fi.resize((*CA_collinear).size());
  for (int i_a = 0; i_a < (*CA_collinear).size(); i_a++){
    CA_ME2_cf[i_a].resize((*CA_collinear)[i_a].size(), 0.);
    CA_ME2_cf_fi[i_a].resize((*CA_collinear)[i_a].size(), 0.);
  }

  CA_value_log_mu2_fact = value_mu_fact;
  if (value_mu_fact[0].size() != 0){
    for (int ss = 0; ss < value_mu_fact[0].size(); ss++){
      CA_value_log_mu2_fact[0][ss] = 2 * log(value_mu_fact[0][ss]);
    }
  }

  if (value_mu_fact[0].size() != 0){
    for (int j = 0; j < value_mu_fact_rel[0].size(); j++){
      logger << LOG_DEBUG << "value_mu_fact[" << setw(2) << 0 << "][" << setw(2) << j << "] = " << setw(25) << value_mu_fact[0][j] << "   CA_value_mu2_fact[" << setw(2) << 0 << "][" << setw(2) << j << "] = " << setw(25) << CA_value_log_mu2_fact[0][j] <<  endline_flush;
    }
  }

  CA_value_ME2_KP.resize(value_mu_fact.size());
  CA_value_pdf_factor.resize(value_mu_fact.size());

  CA_value_integrand.resize(value_mu_fact.size());
  CA_value_integrand_D.resize(value_mu_fact.size());

  CA_sum_value_integrand.resize(value_mu_fact.size());
  CA_sum_value_integrand_D.resize(value_mu_fact.size());

  for (int sd = 0; sd < value_mu_fact.size(); sd++){
    CA_value_ME2_KP[sd].resize(value_mu_fact[sd].size(), vector<vector<double> > (3, vector<double> ((*CA_collinear).size(), 0.)));
    CA_value_pdf_factor[sd].resize(value_mu_fact[sd].size(), vector<vector<vector<double> > > ((*CA_collinear).size(), vector<vector<double> > (3, vector<double> (3, 0.))));

    CA_value_integrand[sd].resize(value_mu_fact[sd].size(), vector<double> ((*CA_collinear).size()));
    CA_value_integrand_D[sd].resize(value_mu_fact[sd].size(), vector<vector<double> > (3, vector<double> ((*CA_collinear).size())));

    CA_sum_value_integrand[sd].resize(value_mu_fact[sd].size());
    CA_sum_value_integrand_D[sd].resize(value_mu_fact[sd].size(), vector<double> (3));
  }

  data_K.resize(3, vector<vector<double> > ((*CA_collinear).size()));
  for (int i_x = 0; i_x < 3; i_x++){
    for (int i_a = 0; i_a < (*CA_collinear).size(); i_a++){
      data_K[i_x][i_a].resize((*CA_collinear)[i_a].size(), 0.);
    }
  }

  value_logscale2_fact_papi.resize(max_dyn_fact + 1);
  value_data_P.resize(max_dyn_fact + 1);
  value_ME2_KP.resize(max_dyn_fact + 1);

  if (switch_TSV){
    value_central_scale_fact[0] = scale_fact / prefactor_reference;
    // !!!    value_central_scale_fact[0] = scale_fact;
    value_central_logscale2_fact[0] = 2 * log(value_central_scale_fact[0]);
    // !!!    value_central_logscale2_fact[0] = 2 * log(scale_fact);

    for (int v_sf = 0; v_sf < max_dyn_fact + 1; v_sf++){  //
      value_logscale2_fact_papi[v_sf].resize(n_scale_dyn_fact[v_sf], vector<vector<double> > (csi->type_parton[0].size(), vector<double> (csi->type_parton[0].size(),  0.)));
      //    value_logscale2_fact_papi[v_sf].resize(n_scale_dyn_fact[v_sf], vector<vector<double> > (_type_parton.size(), vector<double> (_type_parton.size(),  0.)));
      value_data_P[v_sf].resize(n_scale_dyn_fact[v_sf]);
      value_ME2_KP[v_sf].resize(n_scale_dyn_fact[v_sf]);
      for (int i_f = 0; i_f < value_relative_scale_fact[v_sf].size(); i_f++){
	value_data_P[v_sf][i_f].resize(3, vector<vector<double> > ((*CA_collinear).size()));
	for (int i_x = 0; i_x < 3; i_x++){
	  for (int i_a = 0; i_a < (*CA_collinear).size(); i_a++){
	    value_data_P[v_sf][i_f][i_x][i_a].resize((*CA_collinear)[i_a].size(), 0.);
	  }
	}
	value_ME2_KP[v_sf][i_f].resize(3, vector<double> ((*CA_collinear).size(), 0.));
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void observable_set::initialization_RA(){
  Logger logger("observable_set::initialization_RA");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  n_ps = csi->dipole.size();
  n_pc = csi->dipole.size();
  n_pz = 1;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void observable_set::initialization_specific_RA(){
  Logger logger("observable_set::initialization_specific_RA");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  var_A_integrand_cut.resize(n_qTcut, vector<double> (n_scales_CV));
  var_R_integrand_cut.resize(n_qTcut, vector<double> (n_scales_CV));
  var_R_integrand_cut_incl.resize(n_qTcut, vector<double> (n_scales_CV));

  A_integrand_moment_cut.resize(n_qTcut, vector<vector<double> > (n_scales_CV, vector<double> (n_moments)));
  R_integrand_moment_cut.resize(n_qTcut, vector<vector<double> > (n_scales_CV, vector<double> (n_moments)));
  R_integrand_moment_cut_incl.resize(n_qTcut, vector<vector<double> > (n_scales_CV, vector<double> (n_moments)));

  RA_mu_ren.resize(n_ps);
  RA_mu_fact.resize(n_ps);
  RA_alpha_S_reference.resize(n_ps);
  RA_rel_alpha_S.resize(n_ps);
  RA_mu_ren_CV.resize(n_ps, vector<double> (n_scales_CV));
  RA_mu_fact_CV.resize(n_ps, vector<double> (n_scales_CV));
  RA_alpha_S_CV.resize(n_ps, vector<double> (n_scales_CV));
  RA_rel_alpha_S_CV.resize(n_ps, vector<double> (n_scales_CV));
  for (int i_a = 0; i_a < n_ps; i_a++){
    if (dynamic_scale == 0){
      RA_mu_fact[i_a] = var_mu_fact;
      RA_mu_ren[i_a] = var_mu_ren;
      RA_alpha_S_reference[i_a] = var_alpha_S_reference;
      RA_rel_alpha_S[i_a] = var_rel_alpha_S;
    }
    if (dynamic_scale_CV == 0){
      RA_mu_fact_CV[i_a] = var_mu_fact_CV;
      RA_mu_ren_CV[i_a] = var_mu_ren_CV;
      RA_alpha_S_CV[i_a] = var_alpha_S_CV;
      RA_rel_alpha_S_CV[i_a] = var_rel_alpha_S_CV;
    }
  }

  RA_pdf_factor.resize(n_ps, pdf_factor);
  RA_pdf_factor_CV.resize(n_ps, pdf_factor_CV);

  RA_value_mu_fact.resize(n_ps, value_mu_fact);
  RA_value_mu_ren.resize(n_ps, value_mu_ren);
  RA_value_alpha_S.resize(n_ps, value_alpha_S);
  RA_value_factor_alpha_S.resize(n_ps, value_factor_alpha_S);

  RA_ME2.resize(n_ps, 0.);
  RA_ME2_cc.resize(n_ps, 0.);
  RA_ME2_sc.resize(n_ps, 0.);

  var_RA_ME2.resize(n_ps, 0.);
  var_RA_ME2_CV.resize(n_scales_CV, var_RA_ME2);

  RA_ME2_moment.resize(n_moments, vector<double> (n_ps, 0.));
  RA_ME2_moment_CV.resize(n_scales_CV, RA_ME2_moment);

  RA_integrand_moment.resize(n_moments);
  RA_integrand_moment_CV.resize(n_scales_CV, RA_integrand_moment);

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


