#include "header.hpp"

double observable_set::calculate_sigma12_QCD(double & pdf_factor_x1x2){
  static Logger logger("observable_set::calculate_sigma12_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  static double sig12_QCD = 0.;
  if (initial_gg){
    sig12_QCD = -0.5 * A1g * pdf_factor_x1x2;
  }
  else if (initial_qqx){
    sig12_QCD = -0.5 * A1q * pdf_factor_x1x2;
  }
  else if (initial_aa){
    sig12_QCD = 0.;
  }
  else {logger << LOG_FATAL << "No reasonable core process chosen." << endline_flush; exit(1);}

  logger << LOG_DEBUG_VERBOSE << "finished - return " << sig12_QCD << endline_flush;
  return sig12_QCD;
}


double observable_set::calculate_sigma11_QCD(double & pdf_factor_x1x2, double & tH1F_QCD, double & LQ){
  static Logger logger("observable_set::calculate_sigma11_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  static double sig11_QCD = 0.;
  if (initial_gg){
    sig11_QCD =
      - pdf_factor_x1x2 * B1g
      - tH1F_QCD;
    // LQ contribution not implemented yet !!!
    // - pdf_factor_x1x2 * A1g * LQ; // ???
  }
  else if (initial_qqx){
    sig11_QCD =
      - B1q * pdf_factor_x1x2
      - tH1F_QCD
      - pdf_factor_x1x2 * A1q * LQ;
  }
  else if (initial_aa){
    sig11_QCD = 0. ; 
  }
  else {logger << LOG_FATAL << "No reasonable core process chosen." << endline_flush; exit(1);}

  if (QT_finalstate_massive_coloured){
    sig11_QCD += pdf_factor_x1x2 * Gamma1born;
    logger << LOG_DEBUG_VERBOSE << "QT_finalstate_massive_coloured   sig11_QCD = " << sig11_QCD << endline_flush;
  }
  
  logger << LOG_DEBUG_VERBOSE << "sig11_QCD = " << sig11_QCD << endline_flush;

  logger << LOG_DEBUG_VERBOSE << "finished - return " << sig11_QCD << endline_flush;
  return sig11_QCD;
}


double observable_set::calculate_H1_QCD(double pdf_factor_x1x2, double tH1delta_QCD, double tH1_QCD, double tH1F_QCD, double LR, double LF, double LQ){
  static Logger logger("observable_set::calculate_H1_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  double H1full_QCD = 0.;
  if (csi->switch_VTX){H1full_QCD = QT_H1_delta_QCD * pdf_factor_x1x2;}
  // tH1delta_QCD = 0. in this case -> should be unified and used to remove tH1delta_QCD !!!
  if (initial_gg){
    H1full_QCD +=
      + tH1delta_QCD
      + tH1_QCD
      + LF * tH1F_QCD;
    if (!csi->switch_VTX){H1full_QCD += -csi->order_alpha_s_born * beta0 * LR * pdf_factor_x1x2;}
    // LQ-dependent term missing !!!
  }
  else if (initial_qqx){
    H1full_QCD +=
      + tH1delta_QCD
      + tH1_QCD
      + (LF - LQ) * tH1F_QCD
      - LQ * pdf_factor_x1x2 * (B1q + 0.5 * A1q * LQ);
    if (!csi->switch_VTX){H1full_QCD += -csi->order_alpha_s_born * beta0 * LR * pdf_factor_x1x2;}
      // replaces 'QT_finalstate_massive_coloured' implementation by using the alpha_S-order of the underlying Born process
  }
  else if (initial_aa){
    //    H1full_QCD = 0.;
  }
  else {logger << LOG_DEBUG_VERBOSE << "No reasonable core process chosen." << endline_flush;}

  logger << LOG_DEBUG_VERBOSE << setw(40) << "tH1delta_QCD + tH1_QCD" << " = " << tH1delta_QCD + tH1_QCD << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "tH1delta_QCD" << " = " << tH1delta_QCD << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "tH1delta_QCD" << " = " << QT_H1_delta_QCD * pdf_factor_x1x2 << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "tH1delta_QCD direct " << " = " << tH1delta_QCD << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "tH1_QCD" << " = " << tH1_QCD << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "LF * tH1F_QCD" << " = " << LF * tH1F_QCD << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "-" << csi->order_alpha_s_born << " * beta0 * LR * pdf_factor_x1x2" << " = " << -csi->order_alpha_s_born * beta0 * LR * pdf_factor_x1x2 << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "LR" << " = " << LR << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "pdf_factor_x1x2" << " = " << pdf_factor_x1x2 << endline_flush;
  logger << LOG_DEBUG_VERBOSE << setw(40) << "H1full_QCD" << " = " << H1full_QCD << endline_flush;

  logger << LOG_DEBUG_VERBOSE << "finished - return " << H1full_QCD << endline_flush;
  return H1full_QCD;
}


double observable_set::calculate_sigma24_QCD(double & pdf_factor_x1x2){
  static Logger logger("observable_set::calculate_sigma24_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  static double sig24_QCD = 0.;
  if (initial_gg){
    sig24_QCD = 1. / 8 * A1g * A1g * pdf_factor_x1x2;
  }
  else if (initial_qqx){
    sig24_QCD = 1. / 8 * A1q * A1q * pdf_factor_x1x2;
  }
  // added again !!!
  else if (initial_aa){
    sig24_QCD = 0.;
  }
  else {logger << LOG_FATAL << "No reasonable core process chosen." << endline_flush; exit(1);}

  logger << LOG_DEBUG_VERBOSE << "finished - return " << sig24_QCD << endline_flush;
  return sig24_QCD;
}


double observable_set::calculate_sigma23_QCD(double & pdf_factor_x1x2, double & sig11_QCD){
  static Logger logger("observable_set::calculate_sigma23_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  static double sig23_QCD = 0.;
  if (initial_gg){
    sig23_QCD =
      - A1g * beta0 / 3. * pdf_factor_x1x2
      - A1g * 0.5 * sig11_QCD;
  }
  else if (initial_qqx){
    sig23_QCD =
      - beta0 * A1q / 3 * pdf_factor_x1x2
      - 0.5 * A1q * sig11_QCD;
  }
  // added again !!!
  else if (initial_aa){
    sig23_QCD = 0.;
  }
  else {logger << LOG_FATAL << "No reasonable core process chosen." << endline_flush; exit(1);}

  logger << LOG_DEBUG_VERBOSE << "finished - return " << sig23_QCD << endline_flush;
  return sig23_QCD;
}


// no explicit tH1_QCD dependence !!!
double observable_set::calculate_sigma22_QCD(double & pdf_factor_x1x2, double & sig11_QCD, double & tH1_QCD, double & tH1F_QCD, double & H1full_QCD, double & tgaga_QCD, double & LR, double & LF, double & LQ){
  static Logger logger("observable_set::calculate_sigma22_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  static double sig22_QCD = 0.;
  if (initial_gg){
    sig22_QCD =
      - 0.5 * A1g * H1full_QCD
      + 0.5 * (beta0 * A1g * LR - A2g) * pdf_factor_x1x2
      - 0.5 * (B1g - beta0) * sig11_QCD
      + 0.5 * B1g * tH1F_QCD
      + 0.5 * tgaga_QCD;
    // LQ contribution missing !!!
  }
  else if (initial_qqx){
   sig22_QCD =
      - 0.5 * A1q * H1full_QCD
      + 0.5 * (beta0 * A1q * LR - A2q) * pdf_factor_x1x2
      - 0.5 * (B1q - beta0) * sig11_QCD
      + 0.5 * B1q * tH1F_QCD
      + 0.5 * tgaga_QCD;

    // Q dependence
    sig22_QCD -= 0.5 * A1q * LQ * beta0 * pdf_factor_x1x2;
    sig22_QCD += 0.5 * A1q * LQ * tH1F_QCD;
    sig22_QCD -= 0.5 * A1q * LQ * sig11_QCD;
  }
  else {logger << LOG_FATAL << "Wrong process specified." << endline_flush; exit(1);}

  if (QT_finalstate_massive_coloured){
    if (initial_diag || initial_pdf_gq){
      sig22_QCD -= Gamma1born * tH1F_QCD;
    }
    if (initial_diag_gg){
      sig22_QCD += - 0.5 * B1g * Gamma1born * pdf_factor_x1x2;
    }
    if (initial_diag_qqx){
      sig22_QCD += - 0.5 * B1q * Gamma1born * pdf_factor_x1x2;
    }
    if (initial_diag){
      sig22_QCD += + 0.5 * Gamma1squared * pdf_factor_x1x2;
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished - return " << sig22_QCD << endline_flush;
  return sig22_QCD;
}


// B2_A_F should be calculated beforehand !!!
double observable_set::calculate_sigma21_QCD(double & pdf_factor_x1x2, double & sig11_QCD, double & tH1delta, double & tH1_QCD, double & tH1F_QCD, double & H1full_QCD, double & tgaga_QCD, double & tcga_QCD, double & tgamma2_QCD, double & LR, double & LF, double & LQ, double & A_F){
  static Logger logger("observable_set::calculate_sigma21_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  static double B2_A_F = 0.;
  static double sig21_QCD = 0.;
  if (initial_gg){
    // B2_A_F recalculated - function 'calculate_B2_A_F()' could be used !!!
    B2_A_F = B2g + .5 * beta0 * A_F;
    // B2g must be B2(g)_A_F in the following formula (misleading name in counterterm_gg)
    sig21_QCD =
      - beta0 * LR * sig11_QCD
      - B1g * H1full_QCD
      - LF * tgaga_QCD
      - B2_A_F * pdf_factor_x1x2
      + beta0 * (tH1_QCD + tH1delta)
      - tcga_QCD
      - tgamma2_QCD
      - C1ggdelta(A_F) * tH1F_QCD
      - 2 * Delta2gg * pdf_factor_x1x2
      + csi->order_alpha_s_born * beta0 * LR * tH1F_QCD;
    // replaces the following by using the alpha_S-order of the underlying Born process
    //      + 2 * beta0 * LR * tH1F_QCD;
    //20170522:
    // + 2 * beta0 * LR * (B1g * pdf_factor_x1x2  seems doubled
    // (contained in  -B1g * H1full_QCD)
    // org:      + 2 * beta0 * LR * (B1g * pdf_factor_x1x2 + tH1F_QCD);
    // new:      + 2 * beta0 * LR * tH1F_QCD;
  }
  //  else {
  else if (initial_qqx){
    // B2_A_F recalculated - function 'calculate_B2_A_F()' could be used !!!
    B2_A_F = B2q + .5 * beta0 * A_F;
    sig21_QCD =
      - beta0 * LR * sig11_QCD
      - B1q * H1full_QCD
      - LF * tgaga_QCD
      - B2_A_F * pdf_factor_x1x2
      + beta0 * (tH1_QCD + tH1delta)
      - tcga_QCD
      - tgamma2_QCD
      - C1qqdelta(A_F) * tH1F_QCD
      - 2 * Delta2qq * pdf_factor_x1x2
      + csi->order_alpha_s_born * beta0 * LR * tH1F_QCD;
     // replaces 'QT_finalstate_massive_coloured' implementation by using the alpha_S-order of the underlying Born process

    // Q dependence // check if identical !!!
    sig21_QCD += LQ * sig11_QCD * beta0;
    sig21_QCD -= LQ * H1full_QCD * A1q;
    sig21_QCD += LQ * (tgaga_QCD + (B1q + 0.5 * A1q * LQ) * tH1F_QCD);
    sig21_QCD -= LQ * A2q * pdf_factor_x1x2;
    //  sig21_QCD += LQ*(sig11_QCD*(beta0-B1q-0.5*A1q*LQ) - A2q*pdf_factor_x1x2 + A1q*(2*beta0*LR*pdf_factor_x1x2-(tH1_QCD + tH1delta)+(LQ-LF)*tH1F_QCD) + tgaga_QCD);
  }

  if (QT_finalstate_massive_coloured){
    /*
    if (initial_qqx){
      // old comment: check if this part might be missing for qqx - solved ???
      sig21_QCD += 2 * beta0 * LR * tH1F_QCD;  // alpha_S² at LO, not particularly because of mass - but it appears only in ttx production so far !!! ???
    }
    */
    // new implementation (should also work for simultaneous calculation of channels):
    if (initial_diag || initial_pdf_gq){
      sig21_QCD += Gamma1born * (tH1_QCD + LF * tH1F_QCD);
      // old comment: only in the non-diagonal channel, where H1full_QCD does not contain a virtual amplitudes - solved after splitting off the H1full_QCD contribution ???
    }
    if (initial_diag){
      sig21_QCD +=
	- Gamma1born * 2 * beta0 * LR * pdf_factor_x1x2
// Shouldn't this factor 2 become csi->order_alpha_s_born ???
	+ Gamma1loop * pdf_factor_x1x2
	+ Gamma2 * pdf_factor_x1x2;
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished - return " << sig21_QCD << endline_flush;
  return sig21_QCD;
}


double observable_set::calculate_H2_QCD(double pdf_factor_x1x2, double tH1delta_QCD, double tH1_QCD, double tH1F_QCD, double H1full_QCD, double sig11_QCD, double tgaga_QCD, double tcga_QCD, double tgamma2_QCD, double tH2_QCD, double LR, double LF, double LQ){
  static Logger logger("observable_set::calculate_H2_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  double result_H2 = 0.;
  if (initial_gg){
    result_H2 = 1.  //global_factor
      * (tH2_QCD // equivalent in qqx
	 + 0.5 * beta0 * LF * LF * tH1F_QCD // equivalent in qqx
	 + tgamma2_QCD * LF // equivalent in qqx
	 // 20170523: replace the following term by H1full_QCD, analogously to qqbar case:
	 //	 - 2 * beta0 * LR * (tH1_QCD
	 //			     + LF * tH1F_QCD
	 //			     - 2 * beta0 * LR * pdf_factor_x1x2) // to be analyzed...
	 - 2 * beta0 * LR * H1full_QCD // equivalent in qqx
	 // until here
	 - beta0 * LF * LR * tH1F_QCD // LF * LR term in qqx hidden somewhere... 3 times in total (including the part in H1full_QCD ???)
	 + LF * tcga_QCD // equivalent in qqx
	 - beta0 * LR * (tH1_QCD + tH1delta_QCD) // to be analyzed... 3 times in total (including the part in H1full_QCD ???)
	 + 0.5 * LF * LF * tgaga_QCD // equivalent in qqx
	 - 2 * (0.5 * (beta0 * LR) * (beta0 * LR) + beta1 * LR) * pdf_factor_x1x2 // to be analyzed... beta0²*LR² term -3 times in total (including the part in H1full_QCD ???)
	 // Include missing delta term from C*gamma (no factor 2 here !)
	 // xmsq = xmsq + asopi**2*(LF*C1ggdelta*tH1stF)
	 + LF * C1ggdelta(A_F) * tH1F_QCD // equivalent in qqx
	 // Include missing term from contact term in 2 loop AP
	 // xmsq = xmsq + asopi**2*(2*Delta2gg*tdelta)*LF
	 + LF * 2 * Delta2gg * pdf_factor_x1x2); // equivalent in qqx

    logger << LOG_DEBUG_VERBOSE << setw(50) << "LF" << " = " << LF << endline_flush;
    logger << LOG_DEBUG_VERBOSE << setw(50) << "LR" << " = " << LR << endline_flush;
    logger << LOG_DEBUG_VERBOSE << setw(50) << "tH2_QCD" << " = " << tH2_QCD << endline_flush; // different from H2_M_gg !!!
    logger << LOG_DEBUG_VERBOSE << setw(50) << "+ 0.5 * beta0 * LF * LF * tH1F_QCD" << " = " << + 0.5 * beta0 * LF * LF * tH1F_QCD << endline_flush; //ok
    logger << LOG_DEBUG_VERBOSE << setw(50) << "+ tgamma2_QCD * LF" << " = " << + tgamma2_QCD * LF << endline_flush; //ok
    logger << LOG_DEBUG_VERBOSE << setw(50) << "- 2 * beta0 * LR * ((tH1_QCD + tH1delta_QCD) + LF * tH1F_QCD - 2 * beta0 * LR * pdf_factor_x1x2)" << " = " << - 2 * beta0 * LR * ((tH1_QCD + tH1delta_QCD) + LF * tH1F_QCD - 2 * beta0 * LR * pdf_factor_x1x2) << endline_flush; //ok
    logger << LOG_DEBUG_VERBOSE << setw(50) << "- 2 * beta0 * LR * H1full_QCD" << " = " << - 2 * beta0 * LR * H1full_QCD << endline_flush; // to replace the previous term
    logger << LOG_DEBUG_VERBOSE << setw(50) << "- beta0 * LF * LR * tH1F_QCD" << " = " << - beta0 * LF * LR * tH1F_QCD << endline_flush; //ok
    logger << LOG_DEBUG_VERBOSE << setw(50) << "+ LF * tcga_QCD" << " = " << + LF * tcga_QCD << endline_flush; //ok
    logger << LOG_DEBUG_VERBOSE << setw(50) << "- beta0 * LR * (tH1_QCD + tH1delta_QCD)" << " = " << - beta0 * LR * (tH1_QCD + tH1delta_QCD) << endline_flush; //ok

    logger << LOG_DEBUG_VERBOSE << setw(50) << "+ 0.5 * LF * LF * tgaga_QCD" << " = " << + 0.5 * LF * LF * tgaga_QCD << endline_flush; //ok
    logger << LOG_DEBUG_VERBOSE << setw(50) << "- 2 * (0.5 * (beta0 * LR) * (beta0 * LR) + beta1 * LR) * pdf_factor_x1x2" << " = " << - 2 * (0.5 * (beta0 * LR) * (beta0 * LR) + beta1 * LR) * pdf_factor_x1x2 << endline_flush; //ok
    logger << LOG_DEBUG_VERBOSE << setw(50) << "+ LF * C1ggdelta(A_F) * tH1F_QCD" << " = " << + LF * C1ggdelta(A_F) * tH1F_QCD << endline_flush; //ok
    logger << LOG_DEBUG_VERBOSE << setw(50) << "+ 2 * Delta2gg * pdf_factor_x1x2 * LF" << " = " << + 2 * Delta2gg * pdf_factor_x1x2 * LF << endline_flush; //ok
    //    logger << LOG_DEBUG_VERBOSE << setw(50) << "" << " = " <<  << endline_flush;
    //    logger << LOG_DEBUG_VERBOSE << setw(50) << "" << " = " <<  << endline_flush;
  }
  else if (initial_qqx){
    //  else {
    result_H2 = 1.  //global_factor
      * (tH2_QCD
	 + 0.5 * beta0 * pow(LF, 2) * tH1F_QCD
	 + tgamma2_QCD * LF
	 - beta0 * LR * H1full_QCD
	 + (LF - LQ) * tcga_QCD
	 + 0.5 * (LF - LQ) * (LF - LQ) * tgaga_QCD
	 // missing delta terms. Note that tcga_QCD does contain the delta term, so we only need to add it once
	 + (LF - LQ) * C1qqdelta(A_F) * tH1F_QCD
	 + LF * 2 * Delta2qq * pdf_factor_x1x2

	 // Manually added (terms missing because they are zero in colourless case due to csi->order_alpha_s_born = 0) !!!
	 // Insert analogous term to gg channel here:
	 - .5 * csi->order_alpha_s_born * beta0 * LF * LR * tH1F_QCD
	 // adds missing beta0 dependence of LF * C1qqdelta(A_F) * tH1F_QCD ???
	 - .5 * csi->order_alpha_s_born * beta0 * LR * H1full_QCD
	 - .5 * csi->order_alpha_s_born * beta0 * LR * (tH1_QCD + tH1delta_QCD)
	 - csi->order_alpha_s_born * (0.5 * (beta0 * LR) * (beta0 * LR) + beta1 * LR) * pdf_factor_x1x2
	 // until here.

	 ///
	 /// below here: only relevant for resummation:
	 + (1. / 6 * A1q * beta0 * pow(LQ, 3) * pdf_factor_x1x2
	    + 0.5 * pow(LQ, 2) * (A2q * pdf_factor_x1x2
				  + beta0 * sig11_QCD))
	 - LQ * ((B2_A_F + LQ * A2q) * pdf_factor_x1x2
		 - beta0 * (tH1_QCD + tH1delta_QCD)
		 + tgamma2_QCD
		 + 2 * Delta2qq * pdf_factor_x1x2)
	 - 0.5 * LQ * (B1q + 0.5 * A1q * LQ) * (H1full_QCD + (tH1_QCD + tH1delta_QCD))
	 //  should already be included via H1full_QCD. check!!!
	 //  The term was excluded in the non-distribution calculation (already included via H1full_QCD) and should thus probably be erased also at the other places, as done now !!!
	 // 	       + 0.5 * pow(LQ, 2) * (B1q + 0.5 * A1q * LQ) * (B1q + 0.5 * A1q * LQ) * pdf_factor_x1x2
	 - 0.5 * LQ * (LF - LQ) * (B1q + 0.5 * A1q * LQ) * tH1F_QCD);
  }
  else {logger << LOG_DEBUG_VERBOSE << "No reasonable core process chosen." << endline_flush;}

  logger << LOG_DEBUG_VERBOSE << "finished - return " << result_H2 << endline_flush;
  return result_H2;
}
