#include "header.hpp"
#include "gsl/gsl_sf_bessel.h"
#include "gsl/gsl_integration.h"

void observable_set::calculate_coefficients_Itilde(const double xmio, const int order) {
  //    const double Eulergamma=0.577215664902;
  //    const double z2=1.64493406685;
  //    const double z3=1.20205690316;
  //    const double b0_Eulergamma=2*exp(-Eulergamma);

  //  const double z2 = zeta2;
  //  const double z3 = zeta3;
  //const double b0 = b0_Eulergamma;

    double argum = b0_Eulergamma * xmio;
    double logx = log(xmio);

    //    double zbk0,zbk1,zbk2,zbk3;
    double zbk0 = zbesselk0(argum);
    double zbk1 = zbesselk1(argum);
    double zbk2 = zbesselk2(argum);
    double zbk3 = zbesselk3(argum);

    LL[1] = -zbk0 / pow(xmio, 2);
    LL[2] = 2. / pow(xmio, 2) * (zbk0 * logx - zbk1);
    if (order > 1){
      double log2x = pow(logx, 2);
      LL[3] = -3. / pow(xmio, 2) *(zbk0 * (log2x - zeta2) - 2. * zbk1 * logx + zbk2);
      LL[4] = 4. / pow(xmio, 2) *(zbk0 * (pow(logx, 3) - 3. * zeta2 * logx + 2 * zeta3) - 3. * zbk1 * (log2x - zeta2) + 3. * zbk2 * logx - zbk3);
    }
    /*
    LL1=-zbk0/pow(xmio,2);
    LL2= 2.0/pow(xmio,2)*(zbk0*logx-zbk1);
    if (order==2) {
      LL3=-3.0/pow(xmio,2)*(zbk0*(logx*logx-z2)-2*zbk1*logx+zbk2);
      LL4=4.0/pow(xmio,2)*(zbk0*(logx*logx*logx-3*z2*logx+2*z3)-3*zbk1*(logx*logx-z2)+3*zbk2*logx-zbk3);
    }
    */
    /*
    double &LL1, double &LL2, double &LL3, double &LL4;
    LL1 = LL[1];
    LL2 = LL[2];
    LL3 = LL[3];
    LL4 = LL[4];
    */
}


void observable_set::calculate_integrated_Itilde(const int order){
  //void computeItildaIntegrals(double xmin, double xstep, int steps, vector<double> &I1_int, vector<double> &I2_int, vector<double> &I3_int, vector<double> &I4_int){
  static Logger logger("computeItildaIntegrals");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  gsl_integration_workspace * w = gsl_integration_workspace_alloc (10000);

  double result;
  double error;

  gsl_function F;
  F.params = NULL;

  //  double xmax=60;
  //  double xmax=500; // unchanged I1, I2, ...
  //  double xmax=600; // unchanged I1, I2, ...
  //  double xmax = 600.; // unchanged I1, I2, ... // default
  //  double xmax=6.; // unchanged I1, I2, ...
  //  double xmax=20; // minimal changes I1, I2, ...

  //  int_I.resize(5, vector<double> (steps, 0.));
  //  error_I.resize(5, vector<double> (steps, 0.));
  //  vector<vector<double> > int_I(5, vector<double> (steps, 0.));
  //  vector<vector<double> > error_I(5, vector<double> (steps, 0.));

  double xmin = min_qTcut / 100.;
  double xstep = step_qTcut / 100.;
  double xmax = 600.;  //  default starting value ~ inifinity

  // needs to be modified for logarithmic rcut binning (use already available value_qTcut ???) !!!
  for (int i_q = n_qTcut - 1; i_q >= 0; i_q--){
    double xcut = xmin + i_q * xstep;
    if (i_q < n_qTcut - 1){xmax = xmin + (i_q + 1) * xstep;}
      
    F.function = &Itilde1;
    gsl_integration_qags (&F, xcut, xmax, 0, 1.e-7, 10000, w, &result, &error);
    int_I[1][i_q] = result;
    if (i_q < n_qTcut - 1){
      int_I[1][i_q] += int_I[1][i_q + 1];
      error_I[1][i_q] = sqrt(error_I[1][i_q + 1] * error_I[1][i_q + 1] + error * error);
    }
    else {error_I[1][i_q] = error;}

    F.function = &Itilde2;
    gsl_integration_qags (&F, xcut, xmax, 0, 1.e-7, 10000, w, &result, &error);
    int_I[2][i_q] = result;
    if (i_q < n_qTcut - 1){
      int_I[2][i_q] += int_I[2][i_q + 1];
      error_I[2][i_q] = sqrt(error_I[2][i_q + 1] * error_I[2][i_q + 1] + error * error);
    }
    else {error_I[2][i_q] = error;}

    if (order > 1){
      F.function = &Itilde3;
      gsl_integration_qags (&F, xcut, xmax, 0, 1.e-7, 10000, w, &result, &error);
      int_I[3][i_q] = result;
      if (i_q < n_qTcut - 1){
	int_I[3][i_q] += int_I[3][i_q + 1];
	error_I[3][i_q] = sqrt(error_I[3][i_q + 1] * error_I[3][i_q + 1] + error * error);
      }
      else {error_I[3][i_q] = error;}

      F.function = &Itilde4;
      gsl_integration_qag (&F, xcut, xmax, 0, 1.e-8, 10000, 6, w, &result, &error);
      int_I[4][i_q] = result;
      if (i_q < n_qTcut - 1){
	int_I[4][i_q] += int_I[4][i_q + 1];
	error_I[4][i_q] = sqrt(error_I[4][i_q + 1] * error_I[4][i_q + 1] + error * error);
      }
      else {error_I[4][i_q] = error;}
    }
  }

  stringstream temp_ss;
  for (int i_q = 0; i_q < n_qTcut; i_q++){
    double xcut = xmin + i_q * xstep;
    temp_ss << "rcut = " << setw(12) << setprecision(5) << xcut;
    temp_ss << "   I1: " << setprecision(15) << setw(23) << int_I[1][i_q] << " +- " << setprecision(15) << setw(23) << error_I[1][i_q];
    temp_ss << "   I2: " << setprecision(15) << setw(23) << int_I[2][i_q] << " +- " << setprecision(15) << setw(23) << error_I[2][i_q];
    if (order > 1){
      temp_ss << "   I3: " << setprecision(15) << setw(23) << int_I[3][i_q] << " +- " << setprecision(15) << setw(23) << error_I[3][i_q];
      temp_ss << "   I4: " << setprecision(15) << setw(23) << int_I[4][i_q] << " +- " << setprecision(15) << setw(23) << error_I[4][i_q];
    }
    temp_ss << endline;
  }
  logger << LOG_DEBUG << "Precomputing Itilde integrals" << endline << temp_ss.str() << endline_flush;

  /*
  // can be removed after adaptation of interface !!!
  I1_int = int_I[1];
  I2_int = int_I[2];
  if (order > 1){
    I3_int = int_I[3];
    I4_int = int_I[4];
  }
  */
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}
