#include "header.hpp"

void summary_subprocess::initialization_runtime(){
  Logger logger("summary_subprocess::initialization_runtime");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  // Determines for each subprocess (for the selected scales, qTcut values, etc.):
  // error2_time, time_per_event, used_runtime, used_n_event

  error2_time = 0.;
  time_per_event = 0.;
  used_runtime = 0.;
  used_n_event = 0;
  extrapolated_runtime = 0.;

  logger << LOG_DEBUG << "n_seed = " << n_seed << endline_flush;
  N.resize(n_seed, 0);
  result_run.resize(n_seed, 0.);
  deviation_run.resize(n_seed, 0.);

  error2_time_run.resize(n_seed, 0.);
  time_per_event_run.resize(n_seed, 0.);
  used_runtime_run.resize(n_seed, 0.);
  used_n_event_run.resize(n_seed, 0);

  error2_time_group.resize(ycontribution->extended_directory.size(), 0.);
  time_per_event_group.resize(ycontribution->extended_directory.size(), 0.);
  used_runtime_group.resize(ycontribution->extended_directory.size(), 0.);
  used_n_event_group.resize(ycontribution->extended_directory.size(), 0);

  extrapolated_runtime_group.resize(ycontribution->extended_directory.size(), 0.);
  extrapolated_n_event_group.resize(ycontribution->extended_directory.size(), 0.);
  extrapolated_sigma_deviation_group.resize(ycontribution->extended_directory.size(), 0.);
  extrapolated_sigma_normalization_group.resize(ycontribution->extended_directory.size(), 0.);
  extrapolated_sigma_normalization_deviation_group.resize(ycontribution->extended_directory.size(), 0.);

  error2_time_group_run.resize(ycontribution->extended_directory.size());
  time_per_event_group_run.resize(ycontribution->extended_directory.size());
  used_runtime_group_run.resize(ycontribution->extended_directory.size());
  used_n_event_group_run.resize(ycontribution->extended_directory.size());

  result_group_run.resize(ycontribution->extended_directory.size());
  deviation_group_run.resize(ycontribution->extended_directory.size());
  N_group.resize(ycontribution->extended_directory.size());

  for (int i_o = 0; i_o < ycontribution->extended_directory.size(); i_o++){
    logger << LOG_DEBUG << "ycontribution->extended_directory[" << i_o << "].size() = " << ycontribution->extended_directory[i_o].size() << endline_flush;
    error2_time_group_run[i_o].resize(ycontribution->extended_directory[i_o].size(), 0.);
    time_per_event_group_run[i_o].resize(ycontribution->extended_directory[i_o].size(), 0.);
    used_runtime_group_run[i_o].resize(ycontribution->extended_directory[i_o].size(), 0.);
    used_n_event_group_run[i_o].resize(ycontribution->extended_directory[i_o].size(), 0.);

    result_group_run[i_o].resize(ycontribution->extended_directory[i_o].size(), 0.);
    deviation_group_run[i_o].resize(ycontribution->extended_directory[i_o].size(), 0.);
    N_group[i_o].resize(ycontribution->extended_directory[i_o].size(), 0.);
  }

  N_group_max.resize(ycontribution->extended_directory.size(), 0);

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void summary_subprocess::readin_runtime(string result_moment){
  Logger logger("summary_subprocess::readin_runtime");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  initialization_runtime();

  double used_n_event_time = 0.;
  double used_runtime_time = 0.;

  if (osi->switch_TSV){
    // Should actually use "remove_group_run_qTcut" !!!
    remove_run_qTcut.resize(ycontribution->output_n_qTcut);
    for (int i_q = 0; i_q < ycontribution->output_n_qTcut; i_q++){
      remove_run_qTcut[i_q] = remove_group_run_qTcut_TSV[i_q][0];
    }
  }
  else if (osi->switch_CV){remove_run_qTcut = remove_run_qTcut_CV;}
  else {logger << LOG_ERROR << "Should not happen! Run-time determination is not correctly selected." << endline_flush; exit(1);}

  logger << LOG_DEBUG << "ygeneric->directory_runtime_estimate = " << ygeneric->directory_runtime_estimate << endline_flush;
  if (ygeneric->directory_runtime_estimate != ""){
    string result_file = result_moment + "_" + name + ".dat";

    logger << LOG_DEBUG << "ycontribution->directory[0] = " << ycontribution->directory[0] << endline_flush;
    string addtopath = "";
    for (int i_s = ycontribution->directory[0].size() - 1; i_s >= 0; i_s--){
      if (ycontribution->directory[0][i_s] == '/'){
	addtopath = ycontribution->directory[0].substr(0, i_s + 1);
	logger << LOG_DEBUG << "addtopath = " << addtopath << endline_flush;
	break;
      }
    }

    string filename = "../" + addtopath + ygeneric->directory_runtime_estimate + "/result/" + result_file;
    logger << LOG_DEBUG << "filename (time) = " << filename << endline_flush;

    vector<string> readin;
    get_data(filename, readin);
    if (readin.size() > 5){
      //      error2_time_time = atof(readin[readin.size() - 2].c_str());
      //      time_per_event_time = atof(readin[readin.size() - 3].c_str());
      used_runtime_time = atof(readin[readin.size() - 4].c_str());
      used_n_event_time = atoll(readin[0].c_str());
      //      N_time = atoll(readin[0].c_str());
    }
    logger << LOG_INFO << name << ": used_runtime_time = " << setw(15) << setprecision(8) << used_runtime_time << "   used_n_event_time = " << setw(15) << used_n_event_time << endline_flush;
  }


  // New implementation - monitor runtime of standard and extra mappings separately:
  // also needed for "time" part !!!

  for (int i_o = 0; i_o < ycontribution->extended_directory.size(); i_o++){
    for (int i_z = 0; i_z < ycontribution->extended_directory[i_o].size(); i_z++){
      string result_file = result_moment + "_" + name + ".dat";
      logger << LOG_DEBUG_VERBOSE << "result_file = " << result_file << endline_flush;
      string filename = "../" + ycontribution->extended_directory[i_o][i_z] + "/result/" + result_file;
      logger << LOG_DEBUG_VERBOSE << "NEW filename[" << i_o << "][" << i_z << "] = " << filename << endline_flush;

      vector<string> readin;
      get_data(filename, readin);
      if (readin.size() > 5){
	//	error2_n_event_group_run[i_o][i_z] = atof(readin[readin.size() - 1].c_str());
	error2_time_group_run[i_o][i_z] = atof(readin[readin.size() - 2].c_str());
	time_per_event_group_run[i_o][i_z] = atof(readin[readin.size() - 3].c_str());
	used_runtime_group_run[i_o][i_z] = atof(readin[readin.size() - 4].c_str());
	used_n_event_group_run[i_o][i_z] = atoll(readin[0].c_str()); // generated events !!!
	N_group[i_o][i_z] = atoll(readin[0].c_str());
	
	logger << LOG_DEBUG << "SINGLERUN RUNTIME " << name << endline_flush;
	logger << LOG_DEBUG << setw(15) << name << "[" << setw(1) << i_o << "][" << setw(4) << i_z << "]:   " << setw(25) << "time(s)" << " = " << setw(6) << used_runtime_group_run[i_o][i_z] << endline_flush;
	logger << LOG_DEBUG << setw(15) << name << "[" << setw(1) << i_o << "][" << setw(4) << i_z << "]:   " << setw(25) << "n_event" << " = " << setw(15) << used_runtime_group_run[i_o][i_z] / time_per_event_group_run[i_o][i_z] << endline_flush;
	logger << LOG_DEBUG << setw(15) << name << "[" << setw(1) << i_o << "][" << setw(4) << i_z << "]:   " << setw(25) << "time / n_event" << " = " << setw(15) << setprecision(8) << time_per_event_group_run[i_o][i_z] << endline_flush;
	logger << LOG_DEBUG << setw(15) << name << "[" << setw(1) << i_o << "][" << setw(4) << i_z << "]:   " << setw(25) << "error2 x time" << " = " << setw(15) << setprecision(8) << error2_time_group_run[i_o][i_z] << endline_flush;
	logger << LOG_DEBUG << setw(15) << name << "[" << setw(1) << i_o << "][" << setw(4) << i_z << "]:   " << setw(25) << "n_event (generated)" << " = " << setw(15) << used_n_event_group_run[i_o][i_z] << "   (not relevant)" << endline_flush;
	//	logger << LOG_DEBUG << setw(15) << name << "[" << setw(1) << i_o << "][" << setw(4) << i_z << "]:   " << setw(25) << "n_event (generated)" << " = " << setw(15) << setprecision(8) << N_group[i_o][i_z] << "   (not relevant)" << endline_flush;

	//  runtime correction from directory_runtime_estimate (assume same time/event as in time run): !!!
	// used_runtime_time is meant to use used_runtime_per_event only from the time run
	// (could make sense in particular if this run is done with the same hardware and
	// the same conditions for all contributions.
	// It would however change also the used-runtime output to a normalized version of it,
	// thus it is switched off for now:
	logger << LOG_DEBUG << "used_runtime_time = " << used_runtime_time << endline_flush;
	used_runtime_time = 0.;
	if (used_runtime_time != 0.){
	  double new_used_runtime = used_n_event_group_run[i_o][i_z] / used_n_event_time * used_runtime_time;
	  time_per_event_group_run[i_o][i_z] = time_per_event_group_run[i_o][i_z] * (new_used_runtime / used_runtime_group_run[i_o][i_z]);
	  error2_time_group_run[i_o][i_z] = error2_time_group_run[i_o][i_z] * (new_used_runtime / used_runtime_group_run[i_o][i_z]);
	  used_runtime_group_run[i_o][i_z] = new_used_runtime;

	  logger << LOG_DEBUG << name << ": (rescaled) new_used_runtime = " << setw(15) << new_used_runtime << endline_flush;
	}
	double default_deviation = atof(readin[2].c_str());
	logger << LOG_DEBUG << setw(15) << name << "[" << setw(4) << i_z << "]:   " << setw(25) << "default_deviation" << " = " << setw(15) << setprecision(8) << default_deviation << "   (to be changed to reference value)" << endline_flush;
	if (osi->switch_TSV && osi->name_reference_TSV != ""){
	  logger << LOG_DEBUG << "TSV   deviation_group_run[" << i_o << "][" << i_z << "] = " << deviation_group_run[i_o][i_z] << endline_flush;
	  // Apparently, this is not separately read at present (no distinction between different optimizations): use  deviation_run[i_o] instead for now ???
	  // Check how  deviation_group_run[i_o][i_z]  is set !!!
	  if (error2_time_group_run[i_o][i_z] != 0. && default_deviation != 0.){error2_time_group_run[i_o][i_z] = error2_time_group_run[i_o][i_z] * pow(deviation_group_run[i_o][i_z] / default_deviation, 2);}
	}
	else if (osi->switch_CV && ycontribution->active_qTcut && ygeneric->no_qTcut_runtime_estimate){
	  int temp_position = 3 + osi->n_scales_CV * 2 * ygeneric->no_qTcut_runtime_estimate + ((osi->n_scales_CV - 1) / 2) * 2;
	  result_group_run[i_o][i_z] = atof(readin[temp_position].c_str());
	  deviation_group_run[i_o][i_z] = atof(readin[temp_position + 1].c_str());
	  if (error2_time_group_run[i_o][i_z] != 0. && default_deviation != 0.){error2_time_group_run[i_o][i_z] = error2_time_group_run[i_o][i_z] * pow(deviation_group_run[i_o][i_z] / default_deviation, 2);}
	}
	else if (osi->switch_CV){ // unify with previous option ???
	  int temp_position = 3 + ((osi->n_scales_CV - 1) / 2) * 2;
	  result_group_run[i_o][i_z] = atof(readin[temp_position].c_str());
	  deviation_group_run[i_o][i_z] = atof(readin[temp_position + 1].c_str());
	  if (error2_time_group_run[i_o][i_z] != 0. && default_deviation != 0.){error2_time_group_run[i_o][i_z] = error2_time_group_run[i_o][i_z] * pow(deviation_group_run[i_o][i_z] / default_deviation, 2);}
	}
	else {
	  logger << LOG_ERROR << "Should not happen! Run-time determination is not correctly selected." << endline_flush;
	}
      }
      else {
	if (readin.size() != 0){logger << LOG_ERROR << result_file << "   error" << endline_flush;}
	else {logger << LOG_DEBUG << result_file << "   empty" << endline_flush;}
	used_n_event_group_run[i_o][i_z] = 0;
	N_group[i_o][i_z] = 0;
	result_group_run[i_o][i_z] = 0.;
	deviation_group_run[i_o][i_z] = 0.;
	error2_time_group_run[i_o][i_z] = 0.;
	time_per_event_group_run[i_o][i_z] = 0.;
	used_runtime_group_run[i_o][i_z] = 0.;
      }

      string result_file_parameter = "file_parameter." + name + ".dat";
      filename = "../" + ycontribution->extended_directory[i_o][i_z] + "/log/" + result_file_parameter;
      logger << LOG_DEBUG << "filename_parameter." << name << ".dat [" << i_o << "][" << i_z << "] = " << filename << endline_flush;
      // temp_isi actually required ???
      inputparameter_set temp_isi;
      readin.clear();
      temp_isi.parameter_readin_file(filename, readin, false, 1);
      temp_isi.get_userinput_from_readin(readin, temp_isi.input_complete);
      for (int i = temp_isi.input_complete.size() - 1; i >= 0; i--){
	if (temp_isi.input_complete[i].variable == "n_events_max"){
	  long long temp_N = atoll((temp_isi.input_complete[i].value).c_str());
	  if (N_group_max[i_o] < temp_N){N_group_max[i_o] = temp_N;}
	  break;
	}
      }
    }
    
    //  logger << LOG_DEBUG << "Averaging of convergence estimators" << endline_flush;
    int counter_e2_t = 0;
    int counter_t_n = 0;
    error2_time_group[i_o] = 0.;
    time_per_event_group[i_o] = 0.;
    used_runtime_group[i_o] = 0.;
    used_n_event_group[i_o] = 0;

    int this_no_qTcut_runtime_estimate = 0;
    if (ycontribution->active_qTcut){this_no_qTcut_runtime_estimate = ygeneric->no_qTcut_runtime_estimate;}
    if (ycontribution->type_contribution == "PT" || ycontribution->type_contribution == "PT2" || ycontribution->type_contribution == "L2PT"){this_no_qTcut_runtime_estimate = ycontribution->output_n_qTcut - 1;}
    // Simply average over single runs that are not excluded from combination (error2_time, used_n_event):
    //    logger << LOG_DEBUG << "ycontribution->extended_directory[" << i_o << "].size() = " << ycontribution->extended_directory[i_o].size() << endline_flush;
    for (int i_z = 0; i_z < ycontribution->extended_directory[i_o].size(); i_z++){
      if (!remove_run_qTcut[this_no_qTcut_runtime_estimate][i_z]){
	if (error2_time_group_run[i_o][i_z] != 0.){
	  error2_time_group[i_o] += error2_time_group_run[i_o][i_z];
	  counter_e2_t++;
	}
	if (time_per_event_group_run[i_o][i_z] != 0.){
	  time_per_event_group[i_o] += time_per_event_group_run[i_o][i_z];
	  counter_t_n++;
	}
      }
      else {
	logger << LOG_DEBUG << ycontribution->type_contribution << "   " << setw(15) << name << "   run " << i_z << "   removed in time calculation:   remove_run_qTcut[" << this_no_qTcut_runtime_estimate << "][" << i_z << "] = " << remove_run_qTcut[this_no_qTcut_runtime_estimate][i_z] << endline_flush;
      }
      // still count the runtimes and events if run is removed.
      used_runtime_group[i_o] += used_runtime_group_run[i_o][i_z];
      used_n_event_group[i_o] += used_n_event_group_run[i_o][i_z];
    }
    if (counter_e2_t > 0){error2_time_group[i_o] = sqrt(error2_time_group[i_o] / counter_e2_t);}
    if (counter_t_n > 0){time_per_event_group[i_o] = time_per_event_group[i_o] / counter_t_n;}

    /*
    logger << LOG_INFO << "SUMMARY RUNTIME " << name << endline_flush;
    logger << LOG_INFO << setw(15) << name << "[" << setw(1) << i_o << "]:   " << setw(25) << "time(s)" << " = " << setw(6) << used_runtime_group[i_o] << endline_flush;
    logger << LOG_INFO << setw(15) << name << "[" << setw(1) << i_o << "]:   " << setw(25) << "n_event" << " = " << setw(15) << used_runtime_group[i_o] / time_per_event_group[i_o] << endline_flush;
    logger << LOG_INFO << setw(15) << name << "[" << setw(1) << i_o << "]:   " << setw(25) << "time / n_event" << " = " << setw(15) << setprecision(8) << time_per_event_group[i_o] << endline_flush;
    logger << LOG_INFO << setw(15) << name << "[" << setw(1) << i_o << "]:   " << setw(25) << "error2 x time" << " = " << setw(15) << setprecision(8) << error2_time_group[i_o] << endline_flush;
    logger << LOG_INFO << setw(15) << name << "[" << setw(1) << i_o << "]:   " << setw(25) << "n_event (generated)" << " = " << setw(15) << used_n_event_group[i_o] << endline_flush;
    //    logger << LOG_DEBUG << setw(15) << name << "[" << setw(1) << i_o << "]:   " << setw(25) << "n_event (generated)" << " = " << setw(15) << setprecision(8) << N_group[i_o] << "   (not relevant)" << endline_flush;
    */
    // shifted here !!!
    // Replace this part and set error2_time, time_per_event, etc. to [0] values of "group" versions:
    used_runtime = used_runtime_group[0];
    used_n_event = used_n_event_group[0];
    time_per_event = time_per_event_group[0];
    error2_time = error2_time_group[0];
  }
  if (ycontribution->extended_directory.size() == 0){
    // Replace this part and set error2_time, time_per_event, etc. to 0 if no contributions exist:
    used_runtime = 0;
    used_n_event = 0;
    time_per_event = 0.;
    error2_time = 0.;
  }
  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

