#pragma once
#include "2q3y_1l_pmmpp_12345_coefficients.hpp"

template <typename T> class remainder_2q3y_1l_pmmpp_12345 {
	static constexpr size_t NCOEFFS = 23;
	static constexpr size_t NFUNCS = 42;
	static constexpr size_t NLETTERS = 26;

	using TC = std::complex<T>;
	using RCoeffsType = T(*)(const std::array<T,5>&, const std::array<T,NLETTERS>&);
        using FLC = std::vector<std::pair<int,T>>;
        using TCoeffList = std::array<std::pair<FLC, FLC>, NFUNCS>;

	std::shared_ptr<FunctionsManager> f_manager;

	const std::array<RCoeffsType, NCOEFFS> coeff_basis;
	const std::array<std::vector<std::pair<f_type,int>>, NFUNCS> functions_basis;
	// {index of coefficient, index of function, matrix element}
	const TCoeffList coefficients;

  public:
	remainder_2q3y_1l_pmmpp_12345();
	remainder_2q3y_1l_pmmpp_12345(std::shared_ptr<FunctionsManager>);
	
	TC eval(const std::array<T,5>&, int, T);

  private:
        TCoeffList construct_coefficient_list();
};

template <typename T> remainder_2q3y_1l_pmmpp_12345<T>::remainder_2q3y_1l_pmmpp_12345(std::shared_ptr<FunctionsManager> mng) : f_manager(mng),
	coeff_basis{
{eval_coef_2q3y_1l_pmmpp_12345_1<T>,
eval_coef_2q3y_1l_pmmpp_12345_2<T>,
eval_coef_2q3y_1l_pmmpp_12345_3<T>,
eval_coef_2q3y_1l_pmmpp_12345_4<T>,
eval_coef_2q3y_1l_pmmpp_12345_5<T>,
eval_coef_2q3y_1l_pmmpp_12345_6<T>,
eval_coef_2q3y_1l_pmmpp_12345_7<T>,
eval_coef_2q3y_1l_pmmpp_12345_8<T>,
eval_coef_2q3y_1l_pmmpp_12345_9<T>,
eval_coef_2q3y_1l_pmmpp_12345_10<T>,
eval_coef_2q3y_1l_pmmpp_12345_11<T>,
eval_coef_2q3y_1l_pmmpp_12345_12<T>,
eval_coef_2q3y_1l_pmmpp_12345_13<T>,
eval_coef_2q3y_1l_pmmpp_12345_14<T>,
eval_coef_2q3y_1l_pmmpp_12345_15<T>,
eval_coef_2q3y_1l_pmmpp_12345_16<T>,
eval_coef_2q3y_1l_pmmpp_12345_17<T>,
eval_coef_2q3y_1l_pmmpp_12345_18<T>,
eval_coef_2q3y_1l_pmmpp_12345_19<T>,
eval_coef_2q3y_1l_pmmpp_12345_20<T>,
eval_coef_2q3y_1l_pmmpp_12345_21<T>,
eval_coef_2q3y_1l_pmmpp_12345_22<T>,
eval_coef_2q3y_1l_pmmpp_12345_23<T>}
	},

	functions_basis{
{{{f_type::one,1}},
{{f_type::bci,0}},
{{f_type::f,f_manager->add({1,1,1})}},
{{f_type::f,f_manager->add({1,1,3})}},
{{f_type::f,f_manager->add({1,1,6})}},
{{f_type::f,f_manager->add({1,1,7})}},
{{f_type::f,f_manager->add({1,1,8})}},
{{f_type::f,f_manager->add({1,1,10})}},
{{f_type::f,f_manager->add({2,1,1})}},
{{f_type::f,f_manager->add({2,1,4})}},
{{f_type::f,f_manager->add({2,1,8})}},
{{f_type::f,f_manager->add({2,1,9})}},
{{f_type::f,f_manager->add({2,1,10})}},
{{f_type::f,f_manager->add({2,1,11})}},
{{f_type::f,f_manager->add({2,1,13})}},
{{f_type::f,f_manager->add({2,1,14})}},
{{f_type::bci,0}, {f_type::bci,0}},
{{f_type::f,f_manager->add({1,1,1})}, {f_type::f,f_manager->add({1,1,1})}},
{{f_type::f,f_manager->add({1,1,2})}, {f_type::f,f_manager->add({1,1,2})}},
{{f_type::f,f_manager->add({1,1,3})}, {f_type::f,f_manager->add({1,1,3})}},
{{f_type::f,f_manager->add({1,1,6})}, {f_type::f,f_manager->add({1,1,6})}},
{{f_type::f,f_manager->add({1,1,7})}, {f_type::f,f_manager->add({1,1,7})}},
{{f_type::bci,0}, {f_type::f,f_manager->add({1,1,1})}},
{{f_type::bci,0}, {f_type::f,f_manager->add({1,1,2})}},
{{f_type::bci,0}, {f_type::f,f_manager->add({1,1,6})}},
{{f_type::bci,0}, {f_type::f,f_manager->add({1,2,2})}},
{{f_type::bci,0}, {f_type::f,f_manager->add({1,2,8})}},
{{f_type::f,f_manager->add({1,1,1})}, {f_type::f,f_manager->add({1,1,3})}},
{{f_type::f,f_manager->add({1,1,1})}, {f_type::f,f_manager->add({1,1,5})}},
{{f_type::f,f_manager->add({1,1,1})}, {f_type::f,f_manager->add({1,1,7})}},
{{f_type::f,f_manager->add({1,1,1})}, {f_type::f,f_manager->add({1,1,8})}},
{{f_type::f,f_manager->add({1,1,1})}, {f_type::f,f_manager->add({1,1,9})}},
{{f_type::f,f_manager->add({1,1,1})}, {f_type::f,f_manager->add({1,1,10})}},
{{f_type::f,f_manager->add({1,1,2})}, {f_type::f,f_manager->add({1,1,3})}},
{{f_type::f,f_manager->add({1,1,2})}, {f_type::f,f_manager->add({1,1,8})}},
{{f_type::f,f_manager->add({1,1,3})}, {f_type::f,f_manager->add({1,1,5})}},
{{f_type::f,f_manager->add({1,1,3})}, {f_type::f,f_manager->add({1,1,6})}},
{{f_type::f,f_manager->add({1,1,3})}, {f_type::f,f_manager->add({1,1,8})}},
{{f_type::f,f_manager->add({1,1,3})}, {f_type::f,f_manager->add({1,1,10})}},
{{f_type::f,f_manager->add({1,1,6})}, {f_type::f,f_manager->add({1,1,8})}},
{{f_type::f,f_manager->add({1,1,7})}, {f_type::f,f_manager->add({1,1,8})}},
{{f_type::f,f_manager->add({1,1,8})}, {f_type::f,f_manager->add({1,1,9})}}}
	},

	coefficients{ construct_coefficient_list() }
{}

template <typename T> remainder_2q3y_1l_pmmpp_12345<T>::remainder_2q3y_1l_pmmpp_12345() : remainder_2q3y_1l_pmmpp_12345(std::make_shared<FunctionsManager>()) {}

template <typename T> typename remainder_2q3y_1l_pmmpp_12345<T>::TCoeffList remainder_2q3y_1l_pmmpp_12345<T>::construct_coefficient_list() {
    TCoeffList toret;

toret[0] = {{{20, 1}},{{14, 1}}};
toret[1] = {{{0, -1},
{10, -1},
{11, -1}},{{2, -1},
{3, -1}}};
toret[2] = {{{0, 1}},{}};
toret[3] = {{{10, 1}},{{2, 1}}};
toret[4] = {{{0, -1},
{10, -1},
{11, -1},
{21, -1},
{22, -1}},{{2, -1},
{3, -1},
{18, -1},
{19, -1}}};
toret[5] = {{{21, 1}},{{18, 1}}};
toret[6] = {{{11, 1}},{{3, 1}}};
toret[7] = {{{22, 1}},{{19, 1}}};
toret[8] = {{{0, T(-2) / T(3)},
{5, 1},
{13, -1},
{17, -1}},{{1, 1},
{6, -1},
{9, -1}}};
toret[9] = {{{0, T(-2) / T(3)},
{5, 1}},{{1, 1}}};
toret[10] = {{{0, T(2) / T(3)},
{12, -1},
{13, 1},
{15, -2}},{{4, -1},
{6, 1},
{7, -2}}};
toret[11] = {{{0, T(2) / T(3)},
{5, -1},
{12, -1},
{13, 1},
{17, -1}},{{1, -1},
{4, -1},
{6, 1},
{9, -1}}};
toret[12] = {{{15, 2}},{{7, 2}}};
toret[13] = {{{5, -1}},{{1, -1}}};
toret[14] = {{{16, 2}},{{8, 2}}};
toret[15] = {{{13, -1},
{16, 2}},{{6, -1},
{8, 2}}};
toret[16] = {{{12, T(-1) / T(6)},
{16, T(1) / T(3)},
{17, T(-1) / T(6)}},{{4, T(-1) / T(6)},
{8, T(1) / T(3)},
{9, T(-1) / T(6)}}};
toret[17] = {{{12, T(-1) / T(2)},
{17, -1}},{{4, T(-1) / T(2)},
{9, -1}}};
toret[18] = {{{0, T(-1) / T(3)}},{}};
toret[19] = {{{16, 1}},{{8, 1}}};
toret[20] = {{{0, T(1) / T(3)},
{12, T(-1) / T(2)},
{15, -1},
{16, 1}},{{4, T(-1) / T(2)},
{7, -1},
{8, 1}}};
toret[21] = {{{15, 1}},{{7, 1}}};
toret[22] = {{{12, 1}},{{4, 1}}};
toret[23] = {{{0, T(-2) / T(3)}},{}};
toret[24] = {{{0, T(2) / T(3)},
{12, -1},
{15, -2},
{16, 2}},{{4, -1},
{7, -2},
{8, 2}}};
toret[25] = {{{16, -2}},{{8, -2}}};
toret[26] = {{{15, 2}},{{7, 2}}};
toret[27] = {{{17, 1}},{{9, 1}}};
toret[28] = {{{0, T(2) / T(3)},
{5, -1}},{{1, -1}}};
toret[29] = {{{0, T(-2) / T(3)},
{12, 1},
{13, -1}},{{4, 1},
{6, -1}}};
toret[30] = {{{17, 1}},{{9, 1}}};
toret[31] = {{{5, 1}},{{1, 1}}};
toret[32] = {{{13, 1}},{{6, 1}}};
toret[33] = {{{0, T(2) / T(3)},
{5, -1}},{{1, -1}}};
toret[34] = {{{5, 1}},{{1, 1}}};
toret[35] = {{{0, T(-2) / T(3)},
{5, 1}},{{1, 1}}};
toret[36] = {{{13, 1},
{16, -2}},{{6, 1},
{8, -2}}};
toret[37] = {{{17, -1}},{{9, -1}}};
toret[38] = {{{13, -1}},{{6, -1}}};
toret[39] = {{{0, T(-2) / T(3)},
{12, 1},
{13, -1},
{15, 2}},{{4, 1},
{6, -1},
{7, 2}}};
toret[40] = {{{0, T(2) / T(3)},
{12, -1},
{13, 1},
{15, -2}},{{4, -1},
{6, 1},
{7, -2}}};
toret[41] = {{{5, -1}},{{1, -1}}}
    ;

    return toret;
}

template <typename T> std::complex<T> remainder_2q3y_1l_pmmpp_12345<T>::eval(const std::array<T,5>& v_in, int str5, T musq) {
    return eval_remainder<NLETTERS>(v_in, str5, musq, coeff_basis, functions_basis, coefficients, f_manager, v_in[0]);
}

extern template class remainder_2q3y_1l_pmmpp_12345<double>;
#ifdef FivePointAmplitudes_QD_ENABLED 
extern template class remainder_2q3y_1l_pmmpp_12345<FunctionsManager::RHP>;
#endif
