#pragma once
#include "2q3y_2l_pmppp_nf_12345_coefficients.hpp"

template <typename T> class remainder_2q3y_2l_pmppp_nf_12345 {
	static constexpr size_t NCOEFFS = 12;
	static constexpr size_t NFUNCS = 7;
	static constexpr size_t NLETTERS = 26;

	using TC = std::complex<T>;
	using RCoeffsType = T(*)(const std::array<T,5>&, const std::array<T,NLETTERS>&);
        using FLC = std::vector<std::pair<int,T>>;
        using TCoeffList = std::array<std::pair<FLC, FLC>, NFUNCS>;

	std::shared_ptr<FunctionsManager> f_manager;

	const std::array<RCoeffsType, NCOEFFS> coeff_basis;
	const std::array<std::vector<std::pair<f_type,int>>, NFUNCS> functions_basis;
	// {index of coefficient, index of function, matrix element}
	const TCoeffList coefficients;

  public:
	remainder_2q3y_2l_pmppp_nf_12345();
	remainder_2q3y_2l_pmppp_nf_12345(std::shared_ptr<FunctionsManager>);
	
	TC eval(const std::array<T,5>&, int, T);

  private:
        TCoeffList construct_coefficient_list();
};

template <typename T> remainder_2q3y_2l_pmppp_nf_12345<T>::remainder_2q3y_2l_pmppp_nf_12345(std::shared_ptr<FunctionsManager> mng) : f_manager(mng),
	coeff_basis{
{eval_coef_2q3y_2l_pmppp_nf_12345_1<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_2<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_3<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_4<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_5<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_6<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_7<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_8<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_9<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_10<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_11<T>,
eval_coef_2q3y_2l_pmppp_nf_12345_12<T>}
	},

	functions_basis{
{{{f_type::one,1}},
{{f_type::f,f_manager->add({1,1,2})}},
{{f_type::f,f_manager->add({1,1,5})}},
{{f_type::f,f_manager->add({1,1,6})}},
{{f_type::f,f_manager->add({1,1,7})}},
{{f_type::f,f_manager->add({1,1,9})}},
{{f_type::f,f_manager->add({1,1,10})}}}
	},

	coefficients{ construct_coefficient_list() }
{}

template <typename T> remainder_2q3y_2l_pmppp_nf_12345<T>::remainder_2q3y_2l_pmppp_nf_12345() : remainder_2q3y_2l_pmppp_nf_12345(std::make_shared<FunctionsManager>()) {}

template <typename T> typename remainder_2q3y_2l_pmppp_nf_12345<T>::TCoeffList remainder_2q3y_2l_pmppp_nf_12345<T>::construct_coefficient_list() {
    TCoeffList toret;

toret[0] = {{{6, 1}},{{0, 1}}};
toret[1] = {{{7, 1}},{{1, 1}}};
toret[2] = {{{8, 1}},{{2, 1}}};
toret[3] = {{{6, T(-6) / T(19)},
{7, -1},
{8, -1},
{9, -1},
{10, -1},
{11, -1}},{{0, T(-6) / T(19)},
{1, -1},
{2, -1},
{3, -1},
{4, -1},
{5, -1}}};
toret[4] = {{{9, 1}},{{3, 1}}};
toret[5] = {{{10, 1}},{{4, 1}}};
toret[6] = {{{11, 1}},{{5, 1}}}
    ;

    return toret;
}

template <typename T> std::complex<T> remainder_2q3y_2l_pmppp_nf_12345<T>::eval(const std::array<T,5>& v_in, int str5, T musq) {
    return eval_remainder<NLETTERS>(v_in, str5, musq, coeff_basis, functions_basis, coefficients, f_manager, v_in[0]);
}

extern template class remainder_2q3y_2l_pmppp_nf_12345<double>;
#ifdef FivePointAmplitudes_QD_ENABLED 
extern template class remainder_2q3y_2l_pmppp_nf_12345<FunctionsManager::RHP>;
#endif
